import io
from pathlib import Path
from ruamel.yaml import YAML
from ruamel.yaml.comments import CommentedMap
from ruamel.yaml.representer import RepresenterError
from ruamel.yaml.scalarstring import PreservedScalarString
import logging, sys
import pickle
from capanno_utils.helpers.string_tools import get_shortened_id
from capanno_utils.helpers.file_management import dump_dict_to_yaml_output

logging.basicConfig(stream=sys.stderr)
logger = logging.getLogger(__name__)
logger.setLevel(logging.WARNING)

cwl_types = ('null', 'boolean', 'int', 'long', 'float', 'double', 'string', 'File', 'Directory')

class CommandLineToolMixin:
    """Mixin methods for working with cwl_classes.CommandLineTool objects. These objects should be preprocessed and
    validated before using these methods"""

    def get_input_parameter_by_short_id(self, id_):
        input_parameter = None
        for command_input_parameter in self.inputs:
            if get_shortened_id(command_input_parameter.id) == id_:
                input_parameter = command_input_parameter
                break
        return input_parameter

    def get_output_parameter_by_short_id(self, id_):
        output_parameter = None
        for command_output_parameter in self.outputs:
            if get_shortened_id(command_output_parameter.id) == id_:
                output_parameter = command_output_parameter
                break
        return output_parameter

    def get_schema_def_requirement(self):
        """
        Checks for SchemaDefRequirment in requirements, and if it is there returns it.
        :return: SchemaDefRequirement | None
        """
        schema_def_requirement = None
        if self.requirements:
            for requirement in self.requirements:
                try:
                    requirement.__getattribute__('types')  # Only the SchemaDefRequirment has the types attribute.
                    schema_def_requirement = requirement
                except AttributeError:
                    pass  # Requirement is not SchemaDefRequirement. Ignore it.
        return schema_def_requirement

    @staticmethod
    def _make_sorted_input_names(inputs_list):
        """
        Sort inputs according to their command line binding order.
        :return: list of input ids (names) sorted by command line binding order order.
        :rtype: list
        """
        sort_key_dict = {}
        for input in inputs_list:
            command_line_binding = input.inputBinding
            if not command_line_binding:  # Will not appear on command line.
                continue
            position = command_line_binding.position  # Will be None if not specified.
            if not position:
                position = 999
            input_id = get_shortened_id(input.id)
            sort_key = [position, input_id]
            sort_key_dict[input_id] = sort_key
        sorted_ids = sorted(sort_key_dict.keys(),
                            key=lambda sort_key: (sort_key_dict[sort_key][0], sort_key_dict[sort_key][1]))
        return sorted_ids

    def get_sorted_inputs_dict(self):
        """
        Transform CommandLineTool.inputs into a nested dict attribute_value sorted based on CommandLineBinding.
        CommandInputParameter objects cannot be represented by yaml (I could register it I guess???).

        """
        clt_inputs_list = self.inputs
        unsorted_inputs_dict = {}
        sort_keys_list = self._make_sorted_input_names(clt_inputs_list)
        schema_def_requirement = self.get_schema_def_requirement()
        for command_input in clt_inputs_list:
            input_id = get_shortened_id(command_input.id)
            ordered_input_field = command_input.get_ordered_input_map(schema_def_requirement)
            unsorted_inputs_dict[input_id] = ordered_input_field
        sorted_inputs_dict = CommentedMap((input_name, unsorted_inputs_dict[input_name]) for input_name in sort_keys_list)
        return sorted_inputs_dict

    def get_outputs_dict(self):
        """
        Transform CommandLineTool.outputs into a nested dict.
        """
        outputs_dict = CommentedMap()
        for output in self.outputs:
            output_id = get_shortened_id(output.id)
            outputs_dict[output_id] = output.get_ordered_output()
        return outputs_dict


    def create_cwl_commented_map(self):
        """
        Create a CWL CommentedMap with preferred formatting from CommandLineTool attribute_value.
        """

        cwl_map = CommentedMap()
        cwl_map['cwlVersion'] = self.cwlVersion
        cwl_map['class'] = 'CommandLineTool'
        cwl_map['baseCommand'] = self.baseCommand
        if self.requirements:
            cwl_map['requirements'] = [requirement.save() for requirement in self.requirements]
            logger.debug("reqs: {}".format(cwl_map['requirements']))
        if self.hints:
            cwl_map['hints'] = self.hints
            logger.debug("hints: {}".format(cwl_map['hints']))
        optional_class_fields = ['arguments']  # List non-required fields represented as classes that can be handled generically.
        for optional_field_name in optional_class_fields:
            optional_class_values = getattr(self, optional_field_name)
            if type(optional_class_values) is list:
                cwl_map[optional_field_name] = []
                for optional_class_value in optional_class_values:
                    if isinstance(optional_class_value, str):
                        logger.debug("list member noninstance: {}".format(optional_class_value))
                        cwl_map[optional_field_name] += [optional_class_value]
                    else:
                       try:
                           if 'position' in optional_class_value.attrs:  # Only CommandLineBinding has position in attrs. Avoids cirucular import with using isinstance()
                               logger.debug("list member instance: {}".format(optional_class_value))
                               # TODO: deal with this CommandLineBinding effectively
                               # cwl_map[optional_field_name] = optional_class_value.get_ordered_input_binding()
                           else:
                               raise NotImplementedError  # Something else going on that we'll have to deal with.
                       except AttributeError as e:
                                logging.error(f"Need to deal with {optional_class_value} of type {optional_class_value}")
                                raise
            else:
                if optional_class_values:
                    logger.debug("scalar noninstance: {}".format(optional_class_values))
                    cwl_map[optional_field_name] = optional_class_values

            # Non-required fields represented as basic python data types. Order of list determines order in CWL file.
            optional_simple_fields = ['stdin', 'stdout', 'stderr', 'temporaryFailCodes', 'permanentFailCodes',
                                      'successCodes', 'label', 'doc']

        for optional_simple_field in optional_simple_fields:
            optional_field_value = getattr(self, optional_simple_field)
            if optional_field_value:
                cwl_map[optional_simple_field] = optional_field_value
        cwl_map['inputs'] = self.get_sorted_inputs_dict()
        cwl_map['outputs'] = self.get_outputs_dict()

        return cwl_map

    def dump_cwl_str(self):
        """

        :return:
        """
        cwl_yaml = self.create_cwl_commented_map()
        cwl_string = dump_dict_to_yaml_output(cwl_yaml)
        return cwl_string


    def dump_cwl(self, filename):
        """
        Create a formatted CWL file.
        """
        cwl_yaml = self.create_cwl_commented_map()
        logger.debug("cwl: {}".format(cwl_yaml))
        dump_dict_to_yaml_output(cwl_yaml, filename)
        return


class CommandInputParameterMixin:

    def _handle_str_input_type(self, type_, schema_def_requirement):
        """
        Should be the terminating function of walking CommandInputParameter.type fields.
        The value of _type must be a string here.
        Return type if type is CWLType,
        otherwise return type from SchemaDefRequirement dict.
        :param _type:
        :param schema_def_dict:
        :return:
        """
        if type_ in cwl_types:
            type_ = type_
        else:
            schema_def_dict = schema_def_requirement._make_schema_def_dict()
            schema_def_name = get_shortened_id(type_)
            type_ = schema_def_dict[schema_def_name]
        return type_

    def _handle_input_type_field(self, schema_def_requirement):
        """
        Returns the type, replacing any schema_def_requirement types with the actual type. Also removes 'name' keys for array types.
        Does not handle records and enums yet.

        :param schema_def_requirement:
        :return:
        """
        def format_complex_type(complex_type_field):
            """
            Deal with record, enum, and array types here. Convert these to map types (OrderedDict) without extra stuff (name fields)
            :param complex_type_field:
            :return:
            """
            if hasattr(complex_type_field, 'type'):
                complex_type_field_dict = complex_type_field.save()
                if complex_type_field.type in ('array', 'enum'):
                    del complex_type_field_dict['name']

            else:
                complex_type_field_dict = complex_type_field.save()
            return complex_type_field_dict

        type_field = self.type
        if isinstance(type_field, str):
            input_type = self._handle_str_input_type(type_field, schema_def_requirement)

        elif isinstance(type_field, list):  # got a list of types.
            input_type = []
            for _type in type_field:
                if isinstance(_type, str):
                    input_type.append(self._handle_str_input_type(_type, schema_def_requirement))
                else:  # Need to handle record, enum, and array types here.
                    input_type.append(format_complex_type(_type))
        else:
            input_type = format_complex_type(type_field)  # Takes care of CommandInput[Array/Enum/Record]Schmema. Can deal with separately later if we want to.

        return input_type



    @staticmethod
    def _make_base_input_value_field(input_type, default_value, is_optional=False, comment=None):
        """
        Make a value for a single field in an input template. Terminating function for make_input_value_field.
        :param input_type:
        :param default_value:
        :return:
        """
        defaults = {
            'boolean': False,
            'int': None,
            'long': None,
            'float': None,
            'double': None,
            'string': None,
            'File': {'class': 'File', 'path': None, 'location': None},
            'Directory': {'class': 'File', 'path': None, 'location': None}
        }

        if isinstance(input_type, list):
            if 'null' in input_type:
                is_optional = True
                input_type.remove('null')
            if len(input_type) == 1:
                template_param_value, comment = CommandInputParameterMixin._make_base_input_value_field(input_type[0], default_value, is_optional=is_optional, comment=comment)
            else: # Have an input that accepts multiple types. Have not seen this case yet.
               for input_type_entry in input_type:
                   raise NotImplementedError
        elif isinstance(input_type, dict) and 'type' in input_type:  # includes arrays, enums, and records.
            if input_type['type'] == 'array':
                if input_type['items'] in cwl_types:
                    comment = 'array'
                    template_param_value, comment = CommandInputParameterMixin._make_base_input_value_field(input_type['items'], default_value, is_optional=is_optional, comment=comment)
                    if not isinstance(template_param_value, list):
                        template_param_value = [template_param_value]  # Array should be a list. Make it one if not already.

                else:
                    raise NotImplementedError  # Have an array of records or enums or something.
            elif input_type['type'] == 'enum':
                raise NotImplementedError
            elif input_type['type'] == 'record':
                raise NotImplementedError
            else:
                raise NotImplementedError # type could be 'default' or who knows.

        elif input_type in cwl_types:  # Terminating. Should always hit this eventually.
            if not default_value:
                default_value = defaults[input_type]
            template_param_value = default_value
            if is_optional:
                comment = f"Optional {comment if comment else ''} {input_type}"
            else:
                comment = f"Required {comment if comment else ''}{input_type}"
        else:
            raise ValueError(f"Input type is not a cwl type, list, or dict.")




        return template_param_value, comment

    def make_input_value_field(self, schema_def_requirement):
        """
        Make a key: value pair for the input to use in a job file.
        :return:
        """

        default_value = self.default
        input_type = self._handle_input_type_field(schema_def_requirement)  # takes care of schema_def_requirment stuff.
        template_param_value, comment = self._make_base_input_value_field(input_type, default_value)
        return template_param_value, comment



    def get_ordered_input_map(self, schema_def_requirement):
        """
        Turn a CommandInputParameter into a CommentedMap with a consistent order.
        """
        input_map = CommentedMap()
        if self.label:
            input_map['label'] = self.label
        if self.type:  # type is not required for CommandInputParameter. Never seen this case, but...
            input_map['type'] = self._handle_input_type_field(schema_def_requirement)
        if self.default:
            input_map['default'] = self.default
        if self.inputBinding:
            input_map['inputBinding'] = self.inputBinding.get_ordered_input_binding()
        if self.format:
            input_map['format'] = self.format
        if self.streamable:
            input_map['streamable'] = self.streamable
        if self.secondaryFiles:
            input_map['secondaryFiles'] = self.secondaryFiles
        if self.doc:
            input_map['doc'] = PreservedScalarString(self.doc)
        return input_map


class SchemaDefRequirementMixin:

    def _make_schema_def_dict(self):
        """
        Make dictionary from SchemaDefRequirement to populate inputs parameters fully.

        :param schema_def_requirement: SchemaDefRequirement attribute_value.
        :return: keys are input names, values are input parameter fields to drop into inputs section.
        :rtype: dict
        """
        schema_def_types = self.types  # list of InputRecordSchema | InputArraySchema | InputEnumSchema objects.
        if not isinstance(schema_def_types, list):
            raise NotImplementedError
        schema_def_dict = {}
        for type_def in schema_def_types:
            type_def_name = get_shortened_id(type_def.name)
            schema_def_dict[type_def_name] = type_def.save()
            schema_def_dict[type_def_name]['name'] = get_shortened_id(schema_def_dict[type_def_name]['name'])
            if type_def.type == 'record':
                pass
            elif type_def.type == 'array':
                pass
            elif type_def.type == 'enum':
                raise NotImplementedError("EnumRecordSchema puts long id's in for symbols. Make sure this is handled.")
            else:
                raise ValueError(f"Unexpected InputSchema type {repr(type_def)}")
        return schema_def_dict


class CommandLineBindingMixin:
    def get_ordered_input_binding(self):
        """

        :return:
        """
        input_binding = CommentedMap()
        input_binding_fields = ('prefix', 'position', 'loadContents', 'separate', 'itemSeparator', 'valueFrom')
        for binding_field in input_binding_fields:
            binding_field_value = getattr(self, binding_field)
            if binding_field_value:
                input_binding[binding_field] = binding_field_value
        return input_binding


class CommandOutputParameterMixin:


    def _handle_output_type_field(self):

        def format_complex_type(complex_type_field):  # Duplicated from CommandInputParameterMixin._handle_input_type_field. Refactor someday.
            """
            Deal with record, enum, and array types here. Convert these to map types (OrderedDict) without extra stuff (name fields)
            :param complex_type_field:
            :return:
            """
            if hasattr(complex_type_field, 'type'):
                complex_type_field_dict = complex_type_field.save()
                if complex_type_field.type in ('array', 'enum'):
                    del complex_type_field_dict['name']

            else:
                complex_type_field_dict = complex_type_field.save()
            return complex_type_field_dict
        type_field = self.type
        if isinstance(type_field, str):
            try:
                assert type_field in (*cwl_types, 'stdout')
            except AssertionError:
                raise NotImplementedError(f"Need to handle output type {type_field}")
            output_type = type_field
        elif isinstance(type_field, list):
            output_type = []
            for _type in type_field:
                if isinstance(_type, str):
                    try:
                        assert _type in (*cwl_types, 'stdout')
                    except AssertionError:
                        raise NotImplementedError(f"Need to handle output type {_type}")
                    output_type.append(_type)
                else:
                    output_type.append(format_complex_type(_type))
        else:
            output_type = format_complex_type(type_field)
        return output_type

    def get_ordered_output(self):
        output_map = CommentedMap()
        if self.label:
            output_map['label'] = self.label
        if self.type:
            output_map['type'] = self._handle_output_type_field()
        if self.outputBinding:
            output_map['outputBinding'] = self.outputBinding.save()
        if self.format:
            output_map['format'] = self.format
        if self.streamable:
            output_map['streamable'] = self.streamable
        if self.secondaryFiles:
            output_map['secondaryFiles'] = self.secondaryFiles
        if self.doc:
            output_map['doc'] = self.doc
        return output_map


class WorkflowMixin:

    def get_wf_inputs(self):
        wf_inputs = self.inputs
        inputs_dict = CommentedMap()
        for input in wf_inputs:
            single_input_dict = input.to_dict_with_id_key()
            inputs_dict.update(single_input_dict)
        return inputs_dict

    def get_wf_outputs(self):
        wf_outputs = self.outputs
        outputs_dict = CommentedMap()
        for output in wf_outputs:
            individual_output_dict = output.to_dict_with_id_key()  # output is instance of WorkflowOutputParameter
            outputs_dict.update(individual_output_dict)
        return outputs_dict

    def get_wf_steps(self):
        wf_steps = self.steps
        steps_dict = CommentedMap()
        for step in wf_steps:
            individual_step_dict = step.to_dict_with_id_key()
            steps_dict.update(individual_step_dict)
        return steps_dict

    def get_input_parameter_by_short_id(self, id_):
        for input_parameter in self.inputs:
            if get_shortened_id(input_parameter.id) == id_:
                return input_parameter
        return



    def dump_cwl(self, filename):
        cwl_map = CommentedMap()
        cwl_map['cwlVersion'] = self.cwlVersion
        cwl_map['class'] = 'Workflow'
        requirements = self.requirements
        if requirements:
            cwl_map['requirements'] = [requirement.save() for requirement in requirements]
        hints = self.hints
        if hints:
            cwl_map['hints'] = hints  # No idea why hints aren't stored as classes.
        # optional_simple_fields list determines order of these fields if they are present.
        optional_simple_fields = ['label', 'doc']
        for optional_field in optional_simple_fields:
            optional_field_value = getattr(self, optional_field)
            if optional_field_value:
                cwl_map[optional_field] = optional_field_value
        cwl_map['inputs'] = self.get_wf_inputs()  # list of InputParameter objects.
        cwl_map['outputs'] = self.get_wf_outputs()
        cwl_map['steps'] = self.get_wf_steps()
        dump_dict_to_yaml_output(cwl_map, filename)
        return


class InputParameterMixin:

    def to_dict_with_id_key(self):
        input_dict = CommentedMap()
        input_id = get_shortened_id(self.id)
        input_dict[input_id] = CommentedMap()
        input_dict[input_id]['type'] = self.type
        return input_dict

class WorkflowOutputParameterMixin:

    def to_dict_with_id_key(self):
        output_dict = CommentedMap()
        output_id = get_shortened_id(self.id)
        output_dict[output_id] = CommentedMap()
        output_dict[output_id]['type'] = self.type
        output_source_id = self.outputSource
        rel_output_source_id = '/'.join(output_source_id.split('#')[-1].split('/')[1:])
        output_dict[output_id]['outputSource'] = rel_output_source_id
        return output_dict

class WorkflowStepMixin:

    def to_dict_with_id_key(self):
        step_dict = CommentedMap()
        step_id = get_shortened_id(self.id)
        step_dict[step_id] = CommentedMap()
        step_dict[step_id]['run'] = get_shortened_id(self.run)
        step_dict[step_id]['in'] = CommentedMap()
        for step_input in self.in_:  # list of WorkflowStepInput instances.
            step_dict[step_id]['in'].update(step_input.get_workflow_step_inputs())  # WorkflowStepInput class
        # step_dict[step_id]['in'] = [step_input.return_workflow_step_input_dict() for step_input in self.in_]
        step_dict[step_id]['out'] = [get_shortened_id(output) for output in self.out]
        return step_dict

class WorkflowStepInputMixin:

    def get_shortened_source_ids(self): # Might need to rename this if it needs to do more.
        if isinstance(self.source, str):
            source_shortname = get_shortened_id(self.source)
        elif isinstance(self.source, list):
            source_shortname = [get_shortened_id(source) for source in self.source]
        else:
            raise NotImplementedError(f"Deal with WorflowStepInput.source of {self.source}")
        return source_shortname


    def get_workflow_step_inputs(self):
        step_input_dict = CommentedMap()
        input_id = get_shortened_id(self.id)
        step_input_attrs = list(self.attrs)
        step_input_attrs.remove('id')  # Handled explicitly
        step_input_attrs.remove('source')  # Handled explicitly
        step_input_dict[input_id] = CommentedMap()
        step_input_dict[input_id]['source'] = self.get_shortened_source_ids()
        step_input_dict[input_id].update({step_input_attr: getattr(self, step_input_attr) for step_input_attr in step_input_attrs if getattr(self, step_input_attr)})
        return step_input_dict



#
# * This file is subject to the terms and conditions defined in
# * file 'LICENSE.txt', which is part of this source code package.

import os
from pathlib import Path
from ruamel.yaml import safe_load
from capanno_utils.repo_config import config
from tests.test_base import TestBase, test_constants
from capanno_utils.classes.metadata.script_metadata import ScriptMetadata
from capanno_utils.helpers.get_paths import get_cwl_script, get_metadata_path

class TestScriptMetadata(TestBase):

    # def test_make_script_metadata_from_kwargs(self):
    #     kwargs = {'name': 'test_script', 'softwareVersion': test_constants['test_software_version'], 'identifier': 'ST_abcdef.12'}
    #     st_metadata = ScriptMetadata(**kwargs)
    #     self.assertEqual(st_metadata.name, kwargs['name'])
    #     self.assertEqual(st_metadata.identifier, kwargs['identifier'])
    #     return
    #
    # def test_mk_file_from_script_metadata(self):
    #     kwargs = {'name': 'test_script', 'softwareVersion': test_constants['test_software_version'], 'identifier': 'ST_abcdef.12'}
    #     st_metadata = ScriptMetadata(**kwargs)
    #     test_filename = Path(config[os.environ.get('CONFIG_KEY')]['temp_dir'].name) / 'script_test_metadata.yaml'
    #     st_metadata.mk_file(test_filename)
    #     with test_filename.open('r') as file:
    #         test_file_dict = safe_load(file)
    #     self.assertEqual(test_file_dict['identifier'], kwargs['identifier'])
    #     os.remove(test_filename)
    #
    # def test_make_script_metadata_from_file(self):
    #     script_path = get_cwl_script(test_constants['script_group1'], test_constants['script_project1'],test_constants['script_version1'], 'encode_ataqc')
    #     metadata_path = get_metadata_path(script_path)
    #     st_metadata = ScriptMetadata.load_from_file(metadata_path)
    #     return

    # @skip("Pass")
    def test_mk_file_with_inherited_data(self):
        script_cwl_path = get_cwl_script(test_constants['script_group1'], test_constants['script_project1'],test_constants['script_version1'], 'encode_ataqc')
        metadata_path = get_metadata_path(script_cwl_path)
        st_metadata = ScriptMetadata.load_from_file(metadata_path)
        test_filename = Path(config[os.environ.get('CONFIG_KEY')]['temp_dir'].name) / 'script2_test_metadata.yaml'
        st_metadata.mk_completed_file(test_filename)
        with test_filename.open('r') as file:
            test_file_dict = safe_load(file)
        self.assertEqual(test_file_dict['license'], 'MIT')
        os.remove(test_filename)
        return

    # def test_mk_file_without_inherited_data(self):
    #     script_cwl_path = get_cwl_script(test_constants['script_group1'], test_constants['script_project1'],test_constants['script_version1'], 'encode_ataqc')
    #     metadata_path = get_metadata_path(script_cwl_path)
    #     st_metadata = ScriptMetadata.load_from_file(metadata_path)
    #     test_filename = Path(config[os.environ.get('CONFIG_KEY')]['temp_dir'].name) / 'script3_test_metadata.yaml'
    #     st_metadata.mk_file(test_filename)
    #     with test_filename.open('r') as file:
    #         test_file_dict = safe_load(file)
    #     with self.assertRaises(KeyError):
    #         license = test_file_dict['license']
    #     os.remove(test_filename)
    #     return


import os
from tests.test_base import TestBase
from pathlib import Path
from ruamel.yaml import safe_load
from capanno_utils.repo_config import config
from capanno_utils.helpers.get_paths import get_metadata_path
from capanno_utils.validate_content import main

class TestValidateContent(TestBase):

    def test_validate_tool_metadata(self):
        self.update_tool_maps()
        with self.get_content_map_paths()['tool_maps'].open('r') as tm:
            tool_map_dict = safe_load(tm)
        base_path = config[os.environ['CONFIG_KEY']]['base_path']
        for identifier, values in tool_map_dict.items():
            path = base_path / values['metadataPath']

            tool_type = values['type']
            if tool_type == 'parent':
                if not 'common' in path.parts:  # values[type] would be better test.
                    raise ValueError(f"Have a parent tool that is not in a common directory {path}")
            meta_path = path

            main([str(meta_path), '-q'])

        return

    def test_validate_script_metadata(self):
        self.update_script_maps()
        script_map_path = self.get_content_map_paths()['script_maps']
        with script_map_path.open('r') as sm:
            script_map = safe_load(sm)
        base_path = config[os.environ['CONFIG_KEY']]['base_path']
        for script_identifier, script_values in script_map.items():
            metadata_path = base_path / get_metadata_path(Path(script_values['path']))
            main([str(metadata_path), '-q'])
        return


    def test_validate_workflow_metadata(self):
        self.update_workflow_maps()
        with self.get_content_map_paths()['workflow_maps'].open('r') as wm:
            workflow_map = safe_load(wm)
            base_path = config[os.environ['CONFIG_KEY']]['base_path']

        for workflow_identifier, workflow_map_values in workflow_map.items():
            metadata_path = base_path / workflow_map_values['metadataPath']
            main([str(metadata_path), '-q'])
        return


from tempfile import TemporaryDirectory
from tests.test_base import TestBase
from capanno_utils.add.add_workflows import add_workflow

class TestAddWorkflow(TestBase):

    def test_add_workflow(self):
        workflow_group_name = 'test_group'
        workflow_name = 'test_workflow'
        workflow_version = '0.1.1'
        with TemporaryDirectory(prefix='test_add_workflow') as tmp_dir:
            add_workflow(workflow_group_name, workflow_name, workflow_version, root_repo_path=tmp_dir)
            assert True
        return




from urllib import request
from pathlib import Path
from ruamel.yaml import YAML, tokens, error
from ruamel.yaml.comments import CommentedMap
from WDL.CLI import check as check_wdl
from capanno_utils.templates.tool_templates import wdl_templates, snakemake_templates, nextflow_templates
from capanno_utils.helpers.get_paths import get_tool_sources, get_cwl_script, main_tool_subtool_name
from capanno_utils.classes.cwl.common_workflow_language import load_document
import logging, sys

logging.basicConfig(stream=sys.stderr)
logger = logging.getLogger(__name__)
logger.setLevel(logging.WARNING)

blank_line_tk = tokens.CommentToken('\n\n', error.CommentMark(0), None)


def _initialize_command_line_tool_file_yaml(base_command, cwl_path):
    """
    This is kind of dumb. Just make a template file to copy.
    """
    cwl_path = Path(cwl_path)
    cwl_map = CommentedMap()
    doc_placholder = 'Document here.'
    input_name = 'input1'
    output_name = 'output1'
    cwl_map['cwlVersion'] = 'v1.0'
    cwl_map['class'] = 'CommandLineTool'
    cwl_map['baseCommand'] = base_command
    cwl_map['stdout'] = None
    cwl_map['inputs'] = CommentedMap([(input_name, {'type': None, 'inputBinding': {'position': 1, 'prefix': None}, 'doc': doc_placholder})])
    cwl_map['outputs'] = CommentedMap([(output_name, {'type': None, 'outputBinding': {'glob': None}, 'doc': doc_placholder})])
    # insert some blank lines.
    cwl_map.ca.items['stdout'] = [None, None, blank_line_tk, None]
    cwl_map.ca.items['outputs'] = [None, [blank_line_tk], None, None]

    yaml = YAML(pure=True)
    yaml.default_flow_style = False
    yaml.indent(mapping=2, sequence=4, offset=2)
    with cwl_path.open('w') as cwl_file:
        yaml.dump(cwl_map, cwl_file)
    return

def _initialize_empty_wdl_task(wdl_path):
    wdl_path = Path(wdl_path)
    wdl_path.write_text(wdl_templates.wdl_task_template)
    return

def _initialize_empty_sm_rule(sm_path):
    sm_path = Path(sm_path)
    sm_path.write_text(snakemake_templates.snakemake_rule)
    return

def _initialize_empty_nf_tool(nf_path):
    nf_path = Path(nf_path)
    nf_path.write_text(nextflow_templates.nextflow_tool_template)
    return

def _initialize_command_line_tool_from_url(url, cwl_path):

    # response = requests.get(url)
    logger.debug("loading cwl for new subtool {}".format(url))
    clt = load_document(url)
    clt.dump_cwl(cwl_path)
    return

def _initialize_wdl_task_from_url(url, wdl_path):
    logger.debug(f"loading wdl for new subtool/task {url}")
    request.urlretrieve(url, wdl_path)
    check_wdl(path=str(wdl_path)) # check after importing.
    return

def _initialize_sm_rule_from_url(url, sm_path):
    logger.debug(f"loading snakmake for new subtool/task {url}")
    request.urlretrieve(url, sm_path)
    # check_snakemake # Don't have anything to make sure bad things aren't being loaded here.
    return


def _initialize_nf_tool_from_url(url, nf_path):
    logger.debug(f"loading nextflow for new subtool/task {url}")
    request.urlretrieve(url, nf_path)
    # check_nf # Don't have anything to make sure bad things aren't being loaded here.
    return


def initialize_tool_wf_file_tool(tool_name, version_name, subtool_name, init_cwl, init_wdl, init_sm, init_nf, base_dir, ):
    tool_sources = get_tool_sources(tool_name, version_name, subtool_name=subtool_name, base_dir=base_dir)
    cwl_path, wdl_path, sm_path, nf_path = tuple(tool_sources.values())
    if not init_cwl:
        pass
    else:
        if init_cwl == True:
            base_command = f"{tool_name} {subtool_name}" if subtool_name != main_tool_subtool_name else tool_name
            _initialize_command_line_tool_file_yaml(base_command, cwl_path)
        else:
            try:
                assert isinstance(init_cwl, str)  # expect this to be a url.
            except AssertionError:
                print(f"init_cwl is {init_cwl}")
                raise
            _initialize_command_line_tool_from_url(init_cwl, cwl_path)
    if not init_wdl:
        pass
    else:
        if init_wdl == True:
            _initialize_empty_wdl_task(wdl_path)
        else:
            assert isinstance(init_wdl, str)
            _initialize_wdl_task_from_url(init_wdl, wdl_path)
    if not init_sm:
        pass
    else:
        if init_sm == True:
            _initialize_empty_sm_rule(sm_path)
        else:
            _initialize_sm_rule_from_url(init_sm, sm_path)
    if not init_nf:
        pass
    else:
        if init_nf == True:
            _initialize_empty_nf_tool(nf_path)
        else:
            _initialize_nf_tool_from_url(init_nf, nf_path)
    return

def initialize_command_line_tool_file_script(group_name, project_name, script_version, script_name, base_dir):
    cwl_script_path = get_cwl_script(group_name, project_name, script_version, script_name, base_dir)
    base_command = script_name
    _initialize_command_line_tool_file_yaml(base_command, cwl_script_path)
    return



from shutil import copytree
from pathlib import Path
from tempfile import TemporaryDirectory
from tests.test_base import TestBase
from capanno_utils.add.add_tools import add_tool_instance
from capanno_utils.helpers.get_paths import get_tool_version_dir
import capanno_utils.repo_config as config


class TestAddToolInstance(TestBase):

    def test_add_tool_instance(self):
        tool_name = 'cat'
        tool_version = '8.x'
        subtool_name = config.main_tool_subtool_name
        tool_version_dir = get_tool_version_dir(tool_name, tool_version, base_dir=self.test_content_dir)
        with TemporaryDirectory(prefix=f"{type(self).__name__}_") as tmp_dir:
            self.make_empty_tools_index(tmp_dir)
            tool_version_temp_path = Path(tmp_dir) / config.tools_dir_name / tool_name / tool_version
            copytree(tool_version_dir, tool_version_temp_path)  # Copy things to a temp directory so don't screw up real one.
            instance_paths = add_tool_instance(tool_name, tool_version, subtool_name, init_job_file=True,
                                               root_repo_path=tmp_dir)  # returns 2 paths.
            assert True
        return






from shutil import copytree, move
from tempfile import TemporaryDirectory
from pathlib import Path
from unittest import skip
from capanno_utils.repo_config import tools_dir_name
from tests.test_base import TestBase
from capanno_utils.add_content import main as add_content_main
from capanno_utils.helpers.get_paths import *

# @skip('')
class TestAddToolMain(TestBase):
    # @skip('')
    def test_add_tool(self):
        with TemporaryDirectory(prefix='test_add_tool_') as tmp_dir:
            self.make_empty_tools_index(tmp_dir)
            tool_name = 'test_1'
            tool_version = 'fake.1'
            add_content_main(['--no-refresh-index', '-p', tmp_dir, 'tool', tool_name, tool_version, "--has-primary"])
        return

    # @skip('')
    def test_add_tool_with_subtools(self):
        with TemporaryDirectory(prefix='test_add_w_subtools_') as tmp_dir:
            self.make_empty_tools_index(tmp_dir)
            tool_name = 'test_2'
            tool_version = 'fake.2'
            subtools = ['subtool1', 'subtool2', 'subtool3']
            add_content_main(['--no-refresh-index', '-p', tmp_dir, 'tool', tool_name, tool_version] + subtools)
        return

    # @skip('')
    def test_add_tool_with_biotools_id(self):
        with TemporaryDirectory(prefix='with_biotools_') as tmp_dir:
            self.make_empty_tools_index(tmp_dir)
            tool_name = 'test_biotools'
            tool_version = 'fake.3'
            biotools_id = 'malvirus'
            subtools = ['subtool1', 'subtool2', 'subtool3']
            options = ['--biotoolsID', biotools_id]
            add_content_main(['--no-refresh-index', '-p', tmp_dir, 'tool', tool_name, tool_version] + subtools + options)
        return

    def test_add_tool_with_subtools_with_init_urls(self):
        cwl_url = 'https://raw.githubusercontent.com/common-workflow-library/bio-cwl-tools/release/bandage/bandage-image.cwl'
        tool_name = 'test_biotools'
        tool_version = 'fake.3'
        biotools_id = 'malvirus'
        subtools = ['subtool1', 'subtool2', 'subtool3']
        options = ['--biotoolsID', biotools_id]
        init_cwl = ['--init-cwl', f"subtool1={cwl_url}", f"subtool2={cwl_url}"]
        with TemporaryDirectory(prefix='add_with_url') as tmp_dir:
            self.make_empty_tools_index(tmp_dir)
            add_content_main(
                ['--no-refresh-index', '-p', tmp_dir, 'tool', tool_name, tool_version] + subtools + options + init_cwl)

    def test_add_tool_with_subtools_with_init_url_main_only(self):
        cwl_url = 'https://raw.githubusercontent.com/common-workflow-library/bio-cwl-tools/release/bandage/bandage-image.cwl'
        tool_name = 'test_biotools'
        tool_version = 'fake.3'
        biotools_id = 'malvirus'
        options = ['--biotoolsID', biotools_id, '--has-primary']
        init_cwl = ['--init-cwl', cwl_url]
        with TemporaryDirectory(prefix='add_with_url') as tmp_dir:
            self.make_empty_tools_index(tmp_dir)
            add_content_main(
                ['--no-refresh-index', '-p', tmp_dir, 'tool', tool_name, tool_version] + options + init_cwl)

    # @skip('')
    def test_add_subtool(self):
        with TemporaryDirectory(prefix='add_subtool_') as tmp_dir:
            self.make_empty_tools_index(tmp_dir)
            tool_name = 'test_4'
            tool_version = 'fake.4'
            subtool_name = 'new_subtool'
            add_content_main(['--no-refresh-index', '-p', tmp_dir, 'tool', tool_name, tool_version])
            add_content_main(['-p', tmp_dir, 'subtool', tool_name, tool_version, subtool_name, '-u'])
        return

    def test_add_tool_with_existing_cwl_url(self):
        cwl_url = 'https://raw.githubusercontent.com/common-workflow-library/bio-cwl-tools/release/bandage/bandage-image.cwl'
        biotools_id = 'bandage'
        tool_name = 'bandage'
        tool_version = 'fake.6'
        subtool_name = 'image'
        with TemporaryDirectory(prefix='add_tool_with_cwl_') as tmp_dir:
            self.make_empty_tools_index(tmp_dir)
            add_content_main(['--no-refresh-index', '-p', tmp_dir, 'tool', tool_name, tool_version, '--biotoolsID', biotools_id])
            add_content_main(['-p', tmp_dir, 'subtool', tool_name, tool_version, subtool_name, '-u', '--init-cwl', cwl_url])
            assert True
        return

    def test_add_tool_instance(self):
        tool_name = 'STAR'
        tool_version = '2.5'
        subtool_name = 'alignReads'
        tool_directory = get_main_tool_dir(tool_name, base_dir=self.test_content_dir)
        with TemporaryDirectory() as tmp_dir:
            self.make_empty_tools_index(tmp_dir)
            tool_temp_path = Path(tmp_dir) / tools_dir_name / tool_name
            copytree(tool_directory, tool_temp_path)
            add_content_main(['-p', tmp_dir, 'tool-instance', tool_name, tool_version, subtool_name])
            assert True  # Just provides a place for a breakpoint to take a look.at tmp_dir
        return

    def test_add_tool_with_wdl_url(self):
        wdl_url = 'https://github.com/broadinstitute/warp/raw/develop/tasks/skylab/CreateCountMatrix.wdl'
        tool_name = 'who_cares'
        tool_version = 'fake.fake'
        subtool_name = 'test_wdl_subtool'
        with TemporaryDirectory(prefix='add_tool_with_wdl') as tmp_dir:
            self.make_empty_tools_index(tmp_dir)
            add_content_main(
                ['--no-refresh-index', '-p', tmp_dir, 'tool', tool_name, tool_version])
            add_content_main(
                ['-p', tmp_dir, 'subtool', tool_name, tool_version, subtool_name, '-u', '--init-wdl', wdl_url])
            assert True
        return



# @skip('')
class TestAddScriptMain(TestBase):

    def test_add_common_script(self):
        with TemporaryDirectory() as tmp_dir:
            group_name = 'test_group1'
            project_name = 'fake_project_1'
            script_version = '1.nope'
            file_name = "some_filename"
            add_content_main(['-p', tmp_dir, 'common-script', group_name, project_name, script_version, file_name])
        return

    def test_add_script(self):
        with TemporaryDirectory() as tmp_dir:
            group_name = 'test_group2'
            project_name = 'fake_project_2'
            script_name = 'new_script_2'
            script_version = '2.nope'
            add_content_main(['-p', tmp_dir, 'script', group_name, project_name, script_version, script_name])
        return





from tempfile import TemporaryDirectory
from tests.test_base import TestBase
from capanno_utils.add.add_tools import add_tool, add_subtool
from capanno_utils.classes.metadata.tool_metadata import ParentToolMetadata, SubtoolMetadata
from capanno_utils.helpers.get_paths import get_tool_metadata
from capanno_utils.content_maps import make_tools_index
from capanno_utils.exceptions import InIndexError

class TestAddTool(TestBase):

    def test_add_parent_only(self):
        with TemporaryDirectory(prefix='dont_see me_') as tmp_dir:
            self.make_empty_tools_index(tmp_dir)
            tool_name = 'test__1'
            tool_version = '123'
            add_tool(tool_name, tool_version, root_repo_path=tmp_dir, refresh_index=False)  # Index is not refreshed because there is nothing in the content repo. Will throw FileNotFound error when trying to traverse directories.
            # the tool added should be the only identifier in the index.
            parent_metadata_path = get_tool_metadata(tool_name, tool_version, parent=True, base_dir=tmp_dir)
            with self.assertRaises(InIndexError):  # This should fail because trying to assign identifier that is already uses (by itself).
                parent_metadata = ParentToolMetadata.load_from_file(parent_metadata_path, _in_index=False,
                                                                    root_repo_path=tmp_dir)
                assert True
            parent_metadata = ParentToolMetadata.load_from_file(parent_metadata_path, root_repo_path=tmp_dir) # this should succeed with the _in_index flag.
            assert True
        return


    def test_add_with_main(self):
        with TemporaryDirectory() as tmp_dir:
            self.make_empty_tools_index(tmp_dir)
            tool_name = 'test___2'
            tool_version = '123'
            add_tool(tool_name, tool_version, has_primary=True, root_repo_path=tmp_dir, refresh_index=False)
        return


    def test_add_with_subtools(self):
        with TemporaryDirectory() as tmp_dir:
            self.make_empty_tools_index(tmp_dir)
            tool_name = 'test___3'
            tool_version = '123'
            add_tool(tool_name, tool_version, subtool_names=['first', 'second'], has_primary=True, root_repo_path=tmp_dir, refresh_index=False)
            assert True
        return

    def test_add_parent_then_subtool(self):
        with TemporaryDirectory() as tmp_dir:
            self.make_empty_tools_index(tmp_dir)
            tool_name = 'test_tool'
            tool_version = '0.1'
            add_tool(tool_name, tool_version, root_repo_path=tmp_dir, refresh_index=False)
            add_subtool(tool_name, tool_version, 'subtool1', root_repo_path=tmp_dir, update_featureList=True)
            assert True
        return


    def test_add_multiple(self):
        with TemporaryDirectory() as tmp_dir:
            self.make_empty_tools_index(tmp_dir)
            add_tool('fake_1', '123', root_repo_path=tmp_dir, refresh_index=False)  # parent only. Don't try to assemble index. Paths it would need to access aren't there yet.
            add_tool('fake_2', '123', has_primary=True, root_repo_path=tmp_dir)  # # parent with main tool
            add_tool('fake_3', '123', subtool_names=['first', 'second'], has_primary=True,
                     root_repo_path=tmp_dir)
            assert True
        return

    def test_handle_duplicate_subtool_identifier(self):
        """
        Add subtools one at a time.
        :return:
        """
        tool_name = 'samtools'
        tool_version = '1.x'
        subtool_name = 'dict'  # both subtool identifiers start with 'bb'
        subtool_name_2 = 'faidx'
        with TemporaryDirectory() as temp_dir:
            self.make_empty_tools_index(temp_dir)
            add_tool(tool_name, tool_version, subtool_name, root_repo_path=temp_dir, refresh_index=False)
            add_subtool(tool_name, tool_version, subtool_name_2, root_repo_path=temp_dir, update_featureList=True)
            subtool_1_metadata_path = get_tool_metadata(tool_name, tool_version, subtool_name, base_dir=temp_dir)
            subtool_2_metadata_path = get_tool_metadata(tool_name, tool_version, subtool_name_2, base_dir=temp_dir)
            with self.assertRaises(InIndexError):
                subtool_1_metadata = SubtoolMetadata.load_from_file(subtool_1_metadata_path, _in_index=False, root_repo_path=temp_dir)
            subtool_1_metadata = SubtoolMetadata.load_from_file(subtool_1_metadata_path, _in_index=True, root_repo_path=temp_dir)
            subtool_2_metadata = SubtoolMetadata.load_from_file(subtool_2_metadata_path, _in_index=True,
                                                            root_repo_path=temp_dir)
        self.assertNotEqual(subtool_1_metadata.identifier, subtool_2_metadata.identifier)

        return

    def test_handle_duplicate_subtool_identifier_2(self):
        """
        Add two subtools and parent with one command.
        :return:
        """
        tool_name = 'samtools'
        tool_version = '1.x'
        subtool_name = 'dict'  # both subtool identifiers start with 'bb'
        subtool_name_2 = 'faidx'
        with TemporaryDirectory() as temp_dir:
            self.make_empty_tools_index(temp_dir)
            add_tool(tool_name, tool_version, [subtool_name, subtool_name_2], root_repo_path=temp_dir, refresh_index=False)
            subtool_1_metadata_path = get_tool_metadata(tool_name, tool_version, subtool_name, base_dir=temp_dir)
            subtool_2_metadata_path = get_tool_metadata(tool_name, tool_version, subtool_name_2, base_dir=temp_dir)
            subtool_1_metadata = SubtoolMetadata.load_from_file(subtool_1_metadata_path)
            subtool_2_metadata = SubtoolMetadata.load_from_file(subtool_2_metadata_path)
        self.assertNotEqual(subtool_1_metadata.identifier, subtool_2_metadata.identifier)
        return


import os
import unittest
import shutil
from pathlib import Path
from tempfile import NamedTemporaryFile, TemporaryDirectory
from ruamel.yaml import safe_load
from tests.test_base import TestBase, test_constants
from capanno_utils.repo_config import config
from capanno_utils.helpers.get_paths import get_tool_metadata
from capanno_utils.classes.metadata.tool_metadata import ParentToolMetadata, SubtoolMetadata
from capanno_utils.add.add_tools import add_tool, add_subtool


# @unittest.skip('')
class TestMakeParentToolMetadata(TestBase):
    test_dict = {'name': 'parent_name', 'bad': 'A bad key or value.'}

    def test_make_parent_metadata(self):
        p_metadata = ParentToolMetadata(name=TestMakeParentToolMetadata.test_dict['name'], softwareVersion=test_constants['test_software_version'])
        self.assertTrue(p_metadata.name == TestMakeParentToolMetadata.test_dict['name'])

    def test_make_file(self):
        with TemporaryDirectory(prefix='xD_test1', suffix='') as tmpdir:
            p_metadata = ParentToolMetadata(name=TestMakeParentToolMetadata.test_dict['name'],
                                            softwareVersion=test_constants['test_software_version'], check_index=False, root_repo_path=tmpdir)
            self.make_empty_tools_index(tmpdir)
            parent_metadata_path = get_tool_metadata(TestMakeParentToolMetadata.test_dict['name'], test_constants['test_software_version']['versionName'], parent=True, base_dir=tmpdir)
            if not parent_metadata_path.exists():
                parent_metadata_path.parent.mkdir(parents=True)
            p_metadata.mk_file(base_dir=tmpdir, replace_none=True, update_index=True)  # Don't try to update an index file. There isn't one.
            with parent_metadata_path.open('r') as file:
                test_file_dict = safe_load(file)
        self.assertEqual(test_file_dict['name'], TestMakeParentToolMetadata.test_dict['name'])


class TestMakeSubtoolMetadata(TestBase):
    test_dict = {'name': 'subtool_name', 'bad': 'A bad key or value.'}
    def test_make_subtool_metadata(self):
        p_metadata = ParentToolMetadata(name=TestMakeParentToolMetadata.test_dict['name'], softwareVersion=test_constants['test_software_version'], featureList=['subtool_name'])
        st_metadata = SubtoolMetadata(name=TestMakeSubtoolMetadata.test_dict['name'], _parentMetadata=p_metadata)
        self.assertTrue(st_metadata.name == TestMakeSubtoolMetadata.test_dict['name'])

    def test_make_file(self):
        with TemporaryDirectory(prefix="xD_test") as tmpdir:
            self.make_empty_tools_index(tmpdir)
            add_tool('test1', '1.0', 'subtool1', root_repo_path=tmpdir, refresh_index=False)
            assert True
        return


import os
import re
from pathlib import Path
import logging
from typing import List, Set, Dict, Tuple, Optional
from ruamel.yaml import safe_load
from capanno_utils import repo_config as default_cfg
from capanno_utils.repo_config import *


# IDEA: Using Path class, it might be better to generate the get methods by getting longest path, then using Path.parents
# for parent directories. More maintainable if we change path structure. Maybe not.

# Misc

def get_repo_config(base_dir):
    base_dir = Path(base_dir)
    config_dict = default_cfg.config_dict
    repo_config_path = base_dir / 'repo_config.yaml'
    if repo_config_path.exists():
        with repo_config_path.open('r') as f:
            repo_config_dict = safe_load(f)
    else:
        repo_config_dict = {}
        logging.debug(f"No repo specific configuration found for {base_dir}, using defaults.")
    config_dict.update(repo_config_dict) # overwrite default values with repo specific values.
    return config_dict


def get_inputs_schema_template():
    schema_template_path = Path.cwd() / 'tests/test_files/schema_salad/inputs_schema_template.yml'

    return schema_template_path

def get_tools_index(base_dir=None):
    base_dir = get_base_dir(base_dir)
    tools_index = base_dir / tool_index_path


def get_base_dir(base_dir=None):
    if not base_dir:
        base_dir = config[os.environ['CONFIG_KEY']]['base_path']
    return Path(base_dir)


def get_base_dir_from_abs_path(absolute_path, bas_dir_name=content_repo_name):
    absolute_path = Path(absolute_path)
    if not absolute_path.is_absolute():
        raise ValueError(f"{absolute_path} is not an absolute path")
    absolute_path_parts = absolute_path.parts
    if not absolute_path_parts.count(bas_dir_name) == 1:
        raise ValueError(f"Expected the root repo name '{bas_dir_name}' to be path one time.")
    root_path_part_index = absolute_path_parts.index(bas_dir_name)
    base_dir = Path(*absolute_path_parts[:root_path_part_index + 1])
    return base_dir



# tools

def get_root_tools_dir(base_dir=None):
    if not base_dir:
        base_dir = get_base_dir()
    else:
        base_dir = Path(base_dir)
    tool_dir = base_dir / tools_dir_name
    return tool_dir


def get_main_tool_dir(tool_name, base_dir=None):
    root_tools_dir = get_root_tools_dir(base_dir)
    main_tool_dir = root_tools_dir / tool_name
    return main_tool_dir


def get_tool_version_dir(tool_name, tool_version, base_dir=None):
    version_dir = get_main_tool_dir(tool_name, base_dir=base_dir) / str(tool_version)
    return version_dir


def get_tool_dir(tool_name, tool_version, subtool_name=None, base_dir=None):
    tool_version_dir = get_tool_version_dir(tool_name, tool_version, base_dir=base_dir)

    if subtool_name in (None, main_tool_subtool_name):
        tool_dir = tool_version_dir / tool_name
    else:
        tool_dir = tool_version_dir / f"{tool_name}_{subtool_name}"
    return tool_dir


def get_tool_sources(tool_name, tool_version, subtool_name=None, base_dir=None) -> Dict[str,str]:
    """Return path dict for cwl, wdl, sm, and nf files. If subtool_name not specfied, return path dict for main tool."""
    tool_path_dict = {}
    for source_type, extension in [('cwl', 'cwl'),('wdl', 'wdl'),('snakemake', 'snakefile'),('nextflow', 'nf')]:
        if not subtool_name or subtool_name == main_tool_subtool_name:
            tool_dir = get_tool_dir(tool_name, tool_version, subtool_name=main_tool_subtool_name, base_dir=base_dir)
            tool_path_dict[source_type] = tool_dir / f"{tool_name}.{extension}"
        else:
            tool_dir = get_tool_dir(tool_name, tool_version, subtool_name, base_dir=base_dir)
            tool_path_dict[source_type] = tool_dir / f"{tool_name}-{subtool_name}.{extension}"
    return tool_path_dict

def get_tool_sources_from_metadata_path(metadata_path, base_dir=None):
    """
    Return the path dict for cwl, wdl, sm, and nf files from a metadata file path.
    """
    tool_path_dict = {}
    metadata_path = Path(metadata_path)
    metadata_name = metadata_path.name
    if not metadata_file_pattern.match(metadata_name):
        raise ValueError(f"{metadata_path} is not a metadata file or is named incorrectly.")
    if metadata_path.is_absolute():
        pass
    else:
       raise NotImplementedError
    source_path_dir = metadata_path.parent
    source_path_root_name = re.sub(r'-metadata\.ya?ml', '', metadata_name)
    for source_type, extension in [('cwl', 'cwl'),('wdl', 'wdl'),('snakemake', 'snakefile'),('nextflow', 'nf')]:
        tool_path_dict[source_type] = source_path_dir / f"{source_path_root_name}.{extension}"
    return tool_path_dict



def get_tool_common_dir(tool_name, tool_version, base_dir=None):
    version_dir = get_tool_version_dir(tool_name, tool_version, base_dir=base_dir)
    common_dir = version_dir / 'common'
    return common_dir


def get_tool_metadata(tool_name, tool_version, subtool_name=None, parent=False, base_dir=None):
    version_dir = get_tool_version_dir(tool_name, tool_version, base_dir=base_dir)

    if parent:
        assert not subtool_name
        tool_metadata_path = get_tool_common_dir(tool_name, tool_version,
                                                     base_dir=base_dir) / common_tool_metadata_name
    else:
        tool_dir = get_tool_dir(tool_name, tool_version, subtool_name, base_dir)
        if subtool_name in (None, main_tool_subtool_name):
            tool_metadata_path = tool_dir / f"{tool_name}-metadata.yaml"
        else:
            tool_metadata_path = tool_dir / f"{tool_name}-{subtool_name}-metadata.yaml"
    return tool_metadata_path


def get_parent_tool_relative_path_string():
    """Used to populate Subtool.parentMetadata"""
    rel_path_to_parent = f"../../common/{common_tool_metadata_name}"
    return rel_path_to_parent


def get_tool_instances_dir(tool_name, tool_version, subtool_name=None, base_dir=None):
    tool_dir = get_tool_dir(tool_name, tool_version, subtool_name=subtool_name, base_dir=base_dir)
    instances_dir = tool_dir / instances_dir_name
    return instances_dir


def get_tool_instances_dir_from_cwl_path(cwl_path):
    cwl_path = Path(cwl_path)
    instances_dir = cwl_path.parent / instances_dir_name
    return instances_dir

def get_tool_instances_dir_from_metadata_path(metadata_path):
    metadata_path = Path(metadata_path)
    if metadata_path.parent == 'common':
        raise ValueError(f"Parent tools {metadata_path} do not have instance directories. ")
    instances_dir = metadata_path.parent / instances_dir_name
    return instances_dir


def get_tool_cwl_from_instance_path(cwl_instance_path):
    cwl_instance_path = Path(cwl_instance_path)
    tool_instance_path_parts = Path(cwl_instance_path).parts
    tool_name = tool_instance_path_parts[-5]
    tool_version = tool_instance_path_parts[-4]
    subtool_name = tool_instance_path_parts[-3]
    if subtool_name == tool_name:
        subtool_name = main_tool_subtool_name
    subtool_cwl_path = get_tool_sources(tool_name, tool_version, subtool_name)
    return subtool_cwl_path

def get_tool_instance_path(tool_name, tool_version, input_hash, subtool_name=None, base_dir=None):
    tool_inst_dir = get_tool_instances_dir(tool_name, tool_version, subtool_name=subtool_name, base_dir=base_dir)
    inputs_path = tool_inst_dir / f"{input_hash}.yaml"
    return inputs_path

def get_instance_path_from_instance_metadata_path(instance_metadata_path, instance_suffix='.yaml'):
    instance_metadata_path = Path(instance_metadata_path)
    instance_file_name = instance_metadata_path.stem[:-len('-metadata')] + instance_suffix
    tool_instance_path = Path(instance_metadata_path).parent / instance_file_name
    return tool_instance_path


def get_instance_metadata_paths_in_instance_dir(instances_dir):
    instances_dir = Path(instances_dir)
    instance_metadata_files = [metadata_file for metadata_file in instances_dir.iterdir() if
                      metadata_file.name.endswith('-metadata.yaml')]
    return instance_metadata_files

def get_tool_instance_metadata_path(tool_name, tool_version, input_hash, subtool_name=None, base_dir=None):
    tool_inst_dir = get_tool_instances_dir(tool_name, tool_version, subtool_name=subtool_name, base_dir=base_dir)
    instance_metadata_path = tool_inst_dir / f"{input_hash}-metadata.yaml"
    return instance_metadata_path

def get_subtool_metadata_path_from_tool_instance_metadata_path(tool_instance_path, base_dir=None):
    tool_instance_path_parts = Path(tool_instance_path).parts
    tool_name = tool_instance_path_parts[-5]
    tool_version = tool_instance_path_parts[-4]
    subtool_name = tool_instance_path_parts[-3]
    if subtool_name == tool_name:
        subtool_name = main_tool_subtool_name
    subtool_metadata_path = get_tool_metadata(tool_name, tool_version, subtool_name=subtool_name, base_dir=base_dir)
    return subtool_metadata_path


def get_tool_args_from_path(tool_metadata_path):
    tool_metadata_path = Path(tool_metadata_path)
    tool_type = get_tool_type_from_metadata_path(tool_metadata_path)
    path_parts = tool_metadata_path.parts

    tool_name = path_parts[-4]
    tool_version = path_parts[-3]
    subtool_name = path_parts[-2].split('_')[-1] if tool_type == 'subtool' else None

    return tool_name, tool_version, subtool_name


def get_tool_type_from_metadata_path(tool_metadata_path):
    """Not very robust, this one."""
    tool_metadata_path = Path(tool_metadata_path)
    if tool_metadata_path.suffix == '.yaml':
        if tool_metadata_path.parent.parts[-1] == 'common':
            tool_type = "parent"
        else:
            tool_type = "subtool"
    else:
        raise ValueError(f"Do not recognize {tool_metadata_path} as a path to a tool.")
    return tool_type


# scripts

def get_root_scripts_dir(base_dir=None):
    if not base_dir:
        base_dir = get_base_dir()
    else:
        base_dir = Path(base_dir)
    scripts_dir = base_dir / scripts_dir_name
    return scripts_dir


def get_script_group_dir(group_name, base_dir=None):
    scripts_root_dir = get_root_scripts_dir(base_dir=base_dir)
    scripts_group_dir = scripts_root_dir / group_name
    return scripts_group_dir


def get_script_project_dir(group_name, project_name, base_dir=None):
    scripts_group_dir = get_script_group_dir(group_name, base_dir=base_dir)
    script_project_dir = scripts_group_dir / project_name
    return script_project_dir


def get_script_version_dir(group_name, project_name, version, base_dir=None):
    script_ver_dir = get_script_project_dir(group_name, project_name, base_dir=base_dir) / version
    return script_ver_dir


def get_script_dir(group_name, project_name, version, script_name, base_dir=None):
    script_dir = get_script_version_dir(group_name, project_name, version, base_dir=base_dir) / script_name
    return script_dir

def get_cwl_script(group_name, project_name, version, script_name, base_dir=None):
    script_ver_dir = get_script_version_dir(group_name, project_name, version, base_dir=base_dir)
    script_path = script_ver_dir / script_name / f"{script_name}.cwl"
    return script_path


def get_script_metadata(group_name, project_name, version, script_name, base_dir=None):
    script_ver_dir = get_script_version_dir(group_name, project_name, version, base_dir=base_dir)
    script_metadata_path = script_ver_dir / script_name / f"{script_name}-metadata.yaml"
    return script_metadata_path


def get_script_instance_dir(group_name, project_name, version, script_name, base_dir=None):
    script_version_dir = get_script_version_dir(group_name, project_name, version, base_dir=base_dir)
    instances_dir = script_version_dir / script_name / instances_dir_name
    return instances_dir


def get_script_args_from_path(cwl_script_path):
    cwl_script_path = Path(cwl_script_path)
    script_name = cwl_script_path.stem
    path_parts = cwl_script_path.parts
    assert script_name == path_parts[-2]
    script_version = path_parts[-3]
    project_name = path_parts[-4]
    group_name = path_parts[-5]
    return group_name, project_name, script_version, script_name


def get_script_instance_path(group_name, project_name, version, script_name, instance_hash, base_dir=None):
    script_instance_dir = get_script_instance_dir(group_name, project_name, version, script_name, base_dir=base_dir)
    instance_path = script_instance_dir / f"{instance_hash}.yaml"
    return instance_path

def get_script_instance_metadata_path(group_name, project_name, version, script_name, instance_hash, base_dir=None):
    script_instance_dir = get_script_instance_dir(group_name, project_name, version, script_name, base_dir=base_dir)
    instance_metadata_path = script_instance_dir / f"{instance_hash}-metadata.yaml"
    return instance_metadata_path


# workflows

def get_workflows_root_dir(base_dir=None):
    if not base_dir:
        base_dir = get_base_dir()
    else:
        base_dir = Path(base_dir)
    workflows_dir = base_dir / workflows_dir_name
    return workflows_dir


def get_workflow_group_dir(group_name, base_dir=None):
    workflow_group_dir = get_workflows_root_dir(base_dir=base_dir) / group_name
    return workflow_group_dir


def get_workflow_project_dir(group_name, project_name, base_dir=None):
    workflow_project_dir = get_workflow_group_dir(group_name, base_dir=base_dir) / project_name
    return workflow_project_dir


def get_workflow_version_dir(group_name, project_name, version, base_dir=None):
    workflow_ver_dir = get_workflows_root_dir(base_dir=base_dir) / group_name / project_name / version
    return workflow_ver_dir


def get_workflow_sources(group_name, project_name, version, workflow_name, base_dir=None):
    workflow_path_dict = {}
    workflow_ver_dir = get_workflow_version_dir(group_name, project_name, version, base_dir=base_dir)
    for source_type, extension in [('cwl', 'cwl'), ('wdl', 'wdl'), ('snakemake', 'snakefile'), ('nextflow', 'nf')]:
        workflow_path_dict[source_type] = workflow_ver_dir / f"{workflow_name}.{extension}"
    return workflow_path_dict


def get_workflow_sources_from_metadata_path(metadata_path, base_dir=None):
    """
    Return the path dict for cwl, wdl, sm, and nf files from a metadata file path.
    """
    workflow_path_dict = {}
    metadata_path = Path(metadata_path)
    metadata_name = metadata_path.name
    if not metadata_file_pattern.match(metadata_name):
        raise ValueError(f"{metadata_path} is not a metadata file or is named incorrectly.")
    if metadata_path.is_absolute():
        pass
    else:
       raise NotImplementedError
    source_path_dir = metadata_path.parent
    source_path_root_name = re.sub(r'-metadata\.ya?ml', '', metadata_name)
    for source_type, extension in [('cwl', 'cwl'), ('wdl', 'wdl'), ('snakemake', 'snakefile'), ('nextflow', 'nf')]:
        workflow_path_dict[source_type] = source_path_dir / f"{source_path_root_name}.{extension}"
    return workflow_path_dict


def get_workflow_metadata(group_name, project_name, version, base_dir=None):
    workflow_ver_dir = get_workflow_version_dir(group_name, project_name, version, base_dir=base_dir)
    workflow_metadata_path = workflow_ver_dir / f"{project_name}-metadata.yaml"
    return workflow_metadata_path


def get_workflow_instances_dir(group_name, project_name, version, base_dir=None):
    workflow_ver_dir = get_workflow_version_dir(group_name, project_name, version, base_dir=base_dir)
    instances_dir = workflow_ver_dir / instances_dir_name
    return instances_dir


def get_workflow_instance_dir_from_metadata_path(workflow_metadata_path):
    workflow_metadata_path = Path(workflow_metadata_path)
    instances_dir = workflow_metadata_path.parent / instances_dir_name
    return instances_dir


def get_workflow_instance_path(group_name, project_name, version, instance_hash, base_dir=None):
    workflow_instances_dir = get_workflow_instances_dir(group_name, project_name, version, base_dir=base_dir)
    instance_path = workflow_instances_dir / f"{instance_hash}.yaml"
    return instance_path

def get_workflow_instance_metadata(group_name, project_name, version, instance_hash, base_dir=None):
    workflow_instances_dir = get_workflow_instances_dir(group_name, project_name, version, base_dir=base_dir)
    instance_metadata_path = workflow_instances_dir / f"{instance_hash}-metadata.yaml"
    return instance_metadata_path


def get_workflow_args_from_path(workflow_path):
    workflow_path = Path(workflow_path)
    workflow_path_parts = workflow_path.parts
    workflows_dir_index = workflow_path_parts.index(workflows_dir_name)
    workflow_group = workflow_path_parts[workflows_dir_index + 1]
    workflow_project = workflow_path_parts[workflows_dir_index + 2]
    workflow_version = workflow_path_parts[workflows_dir_index + 3]
    return workflow_group, workflow_project, workflow_version


def get_workflow_metadata_from_workflow_path(workflow_path):
    group_name, project_name, version_name = get_workflow_args_from_path(workflow_path)
    base_dir = get_base_dir_from_abs_path(workflow_path)
    workflow_metadata = get_workflow_metadata(group_name, project_name, version_name, base_dir=base_dir)
    return workflow_metadata



# helpers

def get_relative_path(full_path, base_path=None):
    full_path = Path(full_path)
    base_path = get_base_dir(base_dir=base_path)
    return full_path.relative_to(base_path)


def get_metadata_path(cwl_path):
    """
    Naming convention only works for tools and scripts.
    """
    cwl_path = Path(cwl_path)
    path_dir = cwl_path.parent
    metafile_name = f"{cwl_path.stem}-metadata.yaml"
    metadata_path = path_dir / metafile_name
    return metadata_path


def get_file_type_from_main_dir(file_path):
    """
    Determine if a file is a metadata file or a wf file (cwl, wdl, nf, sm) from a subtool, script, or workflow directory.
    Raises an error if it doesn't look like either.
    """
    if file_path.suffix == '.cwl':
        file_type = 'cwl'
    elif file_path.suffix == '.wdl':
        file_type = 'wdl'
    elif file_path.suffix == '.snakefile':
        file_type = 'snakemake'
    elif file_path.suffix == 'nf':
        file_type = 'nextflow'
    elif file_path.suffix in ('.yaml', '.yml'):
        if '-metadata' in file_path.parts[-1]:  # have a metadata file
            file_type = 'metadata'
        else:
            raise ValueError(f"Cannot determine file type for {file_path}")
    else:
        raise ValueError(f"Cannot determine file type for {file_path}")
    return file_type


def get_file_type_from_instance_dir(file_path):
    """
    Determine if a file in an instance directory is an instance file or metadata file.
    Raises and error if it doesn't look like either.
    """
    if instance_file_pattern.match(file_path.name):
        file_type = 'instance'
    elif instance_metadata_file_pattern.match(file_path.name):
        file_type = 'instance_metadata'
    else:
        raise ValueError(f"Cannot determine file type in instances directory: {file_path}")
    return file_type


def check_common_metadata_file_name(file_path, method_type):
    """
    Determine if a file is a properly named common metadata file. If it isn't, raise an error.
    tool_file_name = common-metadata.yaml
    script_common_name = {script_specific}-metadata.yaml
    workflow_common_name = DON'T HAVE COMMON FOR WORKFLOWS YET.
    """
    if method_type == 'workflow':
        raise NotImplementedError(f"Common metadata has not been implemented for workflows yet.")
    elif method_type == 'tool':
        if not file_path.name == common_tool_metadata_name:
            raise ValueError(f"common metadadata files for tools must be named '{common_tool_metadata_name}'")
    elif method_type == 'script':
        if not metadata_file_pattern.match(file_path.name):
            raise ValueError(f"common metadadata files for scripts must be named")
    return


def get_type_from_file_path(abs_path, method_type):
    """
    Determine the type of file from its path.
    abs_path(Path): Path to a file. Should already be sure it is a file path, not a directory.


    """

    if abs_path.parts[-2] == instances_dir_name:
        file_type = get_file_type_from_instance_dir(abs_path)
    elif abs_path.parts[-2] == common_dir_name:
        # in a common directory. Works for tools, scripts, workflows.
        check_common_metadata_file_name(abs_path, method_type)
        file_type = 'common_metadata'
    else:  # Not in common or instances dir. In a subtool, script, or workflow directory.

        file_type = get_file_type_from_main_dir(abs_path)
    return file_type


def get_base_method_type_from_path(abs_path, cwl_root_repo_name=content_repo_name):
    """
    Given a path, determine if the path is in the tools, scripts, workflows, or root directory.
    """
    path_parts = abs_path.parts  # returns tuple.
    if tools_dir_name in path_parts:
        assert scripts_dir_name not in path_parts
        assert workflows_dir_name not in path_parts
        method_type = 'tool'

    elif scripts_dir_name in path_parts:
        assert workflows_dir_name not in path_parts
        method_type = 'script'

    elif workflows_dir_name in path_parts:
        method_type = 'workflow'

    elif path_parts[-1] == cwl_root_repo_name:
        method_type = 'repo_root'
    else:
        raise ValueError(f"{abs_path} does not seem to be a path in a cwl repo.")
    return method_type

def get_dir_type_from_path(abs_dir_path, content_root_repo_name=content_repo_name):
    """
    Get the type of a directory
    Allow optional parameter cwl_root_repo_name in case repo is called something other than `content_repo_name`(capanno at this time).
    Returns
     str: '' | ''
    """
    base_type = get_base_method_type_from_path(abs_dir_path)
    path_parts = abs_dir_path.parts
    if base_type == 'repo_root':
        dir_type = 'base_dir'
    elif base_type == 'tool':
        # dir type could be base_dir, tool_dir, version_dir, common_dir, subtool_dir, instances_dir
        if path_parts[-1] == tools_dir_name:
            assert path_parts[-2] == content_root_repo_name
            dir_type = 'base_dir'
        elif path_parts[-2] == tools_dir_name:
            assert path_parts[-3] == content_root_repo_name
            dir_type = 'tool_dir'
        elif path_parts[-3] == tools_dir_name:
            assert path_parts[-4] == content_root_repo_name
            dir_type = 'version_dir'
        elif path_parts[-4] == tools_dir_name:
            # could be common or main tool
            assert path_parts[-5] == content_root_repo_name
            if path_parts[-1] == 'common':
                dir_type = 'common_dir'
            else:
                dir_type = 'subtool_dir'
        elif path_parts[-5] == tools_dir_name:
            # Should be instances dir
            assert path_parts[-6] == content_root_repo_name
            dir_type = 'instance_dir'
        else:
            raise ValueError
    elif base_type == 'script':
        # dir type could be base_dir, group_dir, project_dir, version_dir, common_dir, script_dir, instances_dir
        if path_parts[-1] == scripts_dir_name:
            assert path_parts[-2] == content_root_repo_name
            dir_type = 'base_dir'
        elif path_parts[-2] == scripts_dir_name:
            assert path_parts[-3] == content_root_repo_name
            dir_type = 'group_dir'
        elif path_parts[-3] == scripts_dir_name:
            assert path_parts[-4] == content_root_repo_name
            dir_type = 'project_dir'
        elif path_parts[-4] == scripts_dir_name:
            assert path_parts[-5] == content_root_repo_name
            dir_type = 'version_dir'
        elif path_parts[-5] == scripts_dir_name:
            assert path_parts[-6] == content_root_repo_name
            # could be script_dir or common_dir.
            if path_parts[-1] == 'common':
                dir_type = 'common_dir'
            else:
                dir_type = 'script_dir'
        elif path_parts[-6] == scripts_dir_name:
            assert path_parts[-7] == content_root_repo_name
            dir_type = 'instance_dir'
        else:
            raise ValueError
    elif base_type == 'workflow':
        if path_parts[-1] == workflows_dir_name:
            assert path_parts[-2] == content_root_repo_name
            dir_type = 'base_dir'
        # import pdb; pdb.set_trace()
        elif path_parts[-2] == workflows_dir_name:
            assert path_parts[-3] == content_root_repo_name
            dir_type = 'group_dir'
        elif path_parts[-3] == workflows_dir_name:
            assert path_parts[-4] == content_root_repo_name
            dir_type = 'project_dir'
        elif path_parts[-4] == workflows_dir_name:
            assert path_parts[-5] == content_root_repo_name
            dir_type = 'version_dir'
        elif path_parts[-5] == workflows_dir_name:
            assert path_parts[-6] == content_root_repo_name
            dir_type = 'group_dir'
        # dir type script_dir, instances_dir
        else:
            raise NotImplementedError
    else:
        raise ValueError

    return base_type, dir_type

def get_types_from_path(path, root_repo_name=content_repo_name, base_path=None):
    """
    Get the type of file from the path.

    path(Path|str): the path of the file.

    return(tuple): ('worklfow' | 'tool' | 'script', 'metadata' | 'cwl')
    """

    path = Path(path)
    if base_path:
        path = Path(base_path) / path
    abs_path = path.resolve()

    method_type = get_base_method_type_from_path(abs_path, cwl_root_repo_name=root_repo_name)

    if method_type == 'repo_root':
        file_type = None

    else:
        if abs_path.is_dir():
            _, file_type = get_dir_type_from_path(abs_path, content_root_repo_name=root_repo_name)
        elif path.is_file():
            file_type = get_type_from_file_path(abs_path, method_type)
        else:
            if not path.exists():
                raise FileNotFoundError(f"{abs_path} does not exist.")
            raise TypeError(f"What kind of path is {abs_path} if it isn't a directory or file?")

    return method_type, file_type





#!/usr/bin/env python3

# Module to call from command line.


import sys
import argparse
from pathlib import Path
from capanno_utils import repo_config
from capanno_utils.add.custom_actions import keyvalueparse
from capanno_utils.add.add_tools import add_tool, add_subtool, add_tool_instance
from capanno_utils.add.add_scripts import add_script, add_common_script_metadata
from capanno_utils.add.add_workflows import add_workflow
import logging

logging.basicConfig(stream=sys.stderr)
logger = logging.getLogger(__name__)
logger.setLevel(logging.WARNING)

def get_parser():
    parser = argparse.ArgumentParser(description='Initialize metadata and directories for a tool, script, or workflow.')
    parser.add_argument('-p','--root-repo-path', dest='root_path', type=Path, default=Path.cwd(), help="Specify the root path of your content repo if it is not the current working directory.")
    parser.add_argument('--no-refresh-index', dest='refresh_index', action='store_false', help="If specified, will use an already made index file that contains identifiers rather than assembling a new one.")
    subparsers = parser.add_subparsers(description='Specify the command to run.', dest='command')

    # add_tool parser
    addtool = subparsers.add_parser('tool', help='Add a new tool to the repository.')
    addtool.add_argument('tool_name', type=str, help="The name of the tool.")
    addtool.add_argument('version_name', type=str, help="The version name of the tool.")
    addtool.add_argument('subtool_names', nargs='*', type=str, help="The names of the subtools/subcommands for the tool. Directories and files will be initialized for the specified subtools.")
    addtool.add_argument('--biotoolsID', type=str, help='biotools id from https://bio.tools. Metadata fields will be populated with data from bio.tools if specified')
    addtool.add_argument('--has-primary', action='store_true', help='Specify if tool is callable without a subcommand.')
    addtool.add_argument('--init-cwl', nargs='*', action=keyvalueparse, help="If specified without arguments, CWL CommandLineTool files will be intiated from a template for the subtools and primary tool if it exists. url's to CWL files for individual subtools may also be provided in a series of arguments in the form subtoolName1=url1.")
    addtool.add_argument('--init-wdl', action='store_true', help="If specified, wdl task files will be intiated for the subtools and primary tool if it exists.")
    addtool.add_argument('--init-nf', action='store_true',
                         help="If specified, nextflow files will be intiated for the subtools and primary tool if it exists.")
    addtool.add_argument('--init-sm', action='store_true',
                         help="If specified, snakemake files will be intiated for the subtools and primary tool if it exists.")
    addtool.add_argument('--no-clobber', action='store_true', help="Do not overwrite the existing tool_name, version_name combination in the repository, if it already exists.", default=False)

    # add_subtool parser
    addsubtool = subparsers.add_parser('subtool', help='Add a subtool to a tool directory. An entry for the tool_name/tool_version must already exist.')
    addsubtool.add_argument('tool_name', help='The name of the tool.')
    addsubtool.add_argument('version_name', help='The version name of the tool.')
    addsubtool.add_argument('subtool_name', help="The name of the subtool to add. The subtool name must be present in the common-metadata featureList field. The featureList field can be updated automatically with the --update-featureList option.")
    addsubtool.add_argument('-u', '--update-featureList', action='store_true', help='Update the featureList of the Application Suite metadata to contain new subtool.')
    addsubtool.add_argument('--init-cwl', nargs='?', type=str, default=False, const=True,
                         help="If specified, CWL CommandLineTool files will be intiated. If a url is provided, the file will be intialized from the url.")
    addsubtool.add_argument('--init-wdl', nargs='?', type=str, default=False, const=True,
                            help="If specified, WDL task file will be intiated . If a url is provided, the file will be intialized from the url.")
    addsubtool.add_argument('--init-nf', nargs='?', type=str, default=False, const=True,
                            help="If specified, a nextflow file will be intiated . If a url is provided, the file will be intialized from the url.")
    addsubtool.add_argument('--init-sm', nargs='?', type=str, default=False, const=True,
                            help="If specified, a snakemake file will be intiated. If a url is provided, the file will be intialized from the url.")
    addsubtool.add_argument('--no-clobber', action='store_true', help="Do not overwrite the existing tool_name, version_name, subtool name combination in the repository, if it already exists.", default=False)

    # add_tool instance parser
    addtoolinstance = subparsers.add_parser('tool-instance', help='Add a tool instance.')
    addtoolinstance.add_argument('tool_name', type=str, help="The name of the tool.")
    addtoolinstance.add_argument('version_name', type=str, help="The version of the tool.")
    addtoolinstance.add_argument('subtool_name', type=str, nargs='?', default=repo_config.main_tool_subtool_name,
                                 help="The subtool name for the instance.")

    # add_common_script_parser
    addscriptcommon = subparsers.add_parser('common-script', help='add script metadata that other scripts can inherit from')
    addscriptcommon.add_argument('group_name', help='The name of the group directory that the script will go into.')
    addscriptcommon.add_argument('project_name', help='The name of the project directory that the script will go into.')
    addscriptcommon.add_argument('script_version', help='The version of the script.')
    addscriptcommon.add_argument('filename', help="The name of common metadata file. File extensions and '-metadata.yaml' postfix must be omitted.")

    # add_script parser
    addscript = subparsers.add_parser('script', help='add a new script to the repository.')
    addscript.add_argument('group_name', help='The name of the group directory that the script will go into.')
    addscript.add_argument('project_name', help='The name of the project directory that the script will go into.')
    addscript.add_argument('script_version', help='The version name of the script.')
    addscript.add_argument('script_name', help='The name of the script. File extensions should be omitted. Replace spaces with underscores')
    addscript.add_argument('--parent-metadata', '-p', nargs='+', help="path(s) to common script metadata that the script should inherit from.")
    addscript.add_argument('--init-cwl', action='store_true',
                         help="If specified, CWL CommandLineTool files will be intiated for the subtools and primary tool if it exists. ")
    # Could add flags for more kwargs here. Probably easiest to just add to file though.

    # add_script instance parser
    addscriptinstance = subparsers.add_parser('script-instance', help='Add a script instance.')
    addscriptinstance.add_argument('group_name', type=str, help="The name of the group directory for the script instance.")
    addscriptinstance.add_argument('project_name', type=str, help="The name of the project directory for the script instance.")
    addscriptinstance.add_argument('version_name', type=str, help="The version of the script.")
    addscriptinstance.add_argument('script_name',
                           help='The name of the script. File extensions should be omitted. Replace spaces with underscores')


    addworkflow = subparsers.add_parser('workflow', help='add a workflow')
    addworkflow.add_argument('group_name', help='The name of the group directory that the workflow will go into.')
    addworkflow.add_argument('workflow_name', help='The name of the workflow. Replace spaces with underscores.')
    addworkflow.add_argument('workflow_version', help='The version of the workflow.')

    return parser


def main(argsl=None):
    if not argsl:
        argsl = sys.argv[1:]
    parser = get_parser()
    args = parser.parse_args(argsl)
    logger.debug("command:{}".format(args.command))

    if args.command == 'tool':
        logger.debug("subtool names:{}".format(args.subtool_names))
        add_tool(args.tool_name, args.version_name, subtool_names=args.subtool_names, biotools_id=args.biotoolsID, has_primary=args.has_primary, init_cwl=args.init_cwl, init_wdl=args.init_wdl, init_sm=args.init_sm, init_nf=args.init_nf, root_repo_path=args.root_path, refresh_index=args.refresh_index, no_clobber=args.no_clobber)
    elif args.command == 'subtool':
        add_subtool(args.tool_name, args.version_name, args.subtool_name, update_featureList=args.update_featureList, init_cwl=args.init_cwl, init_wdl=args.init_wdl, init_sm=args.init_sm, init_nf=args.init_nf, root_repo_path=args.root_path, no_clobber=args.no_clobber)
    elif args.command == 'tool-instance':
        add_tool_instance(args.tool_name, args.version_name, args.subtool_name, root_repo_path=args.root_path)
    elif args.command == 'common-script':
        add_common_script_metadata(args.group_name, args.project_name, args.script_version, args.filename, root_repo_path=args.root_path)
    elif args.command == 'script':
        add_script(args.group_name, args.project_name, args.script_version, args.script_name, parentMetadata=args.parent_metadata, init_cwl=args.init_cwl, root_repo_path=args.root_path)
    elif args.command == 'script-instance':
        raise NotImplementedError
    elif args.command == 'workflow':
        add_workflow(args.group_name, args.workflow_name, args.workflow_version)
    else:
        parser.print_help()
    return

if __name__ == "__main__":
    sys.exit(main(sys.argv[1:]))



from tests.test_base import TestBase
from tempfile import NamedTemporaryFile
from ruamel.yaml import YAML, dump
from capanno_utils.repo_config import config
from capanno_utils.helpers.get_paths import *
from capanno_utils.classes.schema_salad.schema_salad import InputsSchema


class TestMakeCommandLineToolInputsTemplate(TestBase):

    def test_make_command_line_tool_input_template(self):
        tool_name = 'STAR'
        version_name = '2.5'
        subtool_name = 'alignReads'
        tool_sources = get_tool_sources(tool_name, version_name, subtool_name, base_dir=self.test_content_dir)
        inputs_schema = InputsSchema(tool_sources['cwl'])
        template = inputs_schema.make_template()
        yaml = YAML()
        with NamedTemporaryFile(delete=True, prefix=f"{tool_name + subtool_name + version_name}template_", suffix='.yml') as tmp:
            yaml.dump(template, tmp)
            assert True
        return

    def test_make_clt_template_2(self):
        cwl_tool = self.test_files_dir / 'cwl-misc' / 'test_command_line_tool.cwl'
        inputs_schema = InputsSchema(cwl_tool)
        template = inputs_schema.make_template()
        yaml = YAML()
        with NamedTemporaryFile(delete=True, prefix='test_template_', suffix='.yml') as tmp:
            yaml.dump(template, tmp)
        return


from setuptools import setup, find_packages

"""
To release package update `package_version` and commit/push changes, then run
$ python setup.py sdist bdist_wheel
$ twine upload dist/*

for twine upload to work, must have credentials in .pypirc file
"""

with open('README.md', 'r') as fh:
    long_description = fh.read()


package_version = '0.9.6a'

setup(
    name='capanno_utils',
    version=str(package_version),
    packages=find_packages(),
    url='https://github.com/truwl/capanno-utils',
    license='Apache 2.0',
    author='Karl Sebby',
    author_email='karl.sebby@truwl.com',
    description='Tool for managing bioinformatics content repositories.',
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires=[
        'setuptools',
        'requests',
        'ruamel.yaml >= 0.15, <=0.16',
        'semantic-version',
        'pandoc-include',
        'cwltool',
        'miniwdl'
    ],
    entry_points={
        'console_scripts': ["capanno-validate=capanno_utils.validate_content:main",
                            "capanno-add=capanno_utils.add_content:main",
                            "capanno-map=capanno_utils.make_content_maps:main",
                            "capanno-id=capanno_utils.make_ids:main",
                            "capanno-status=capanno_utils.change_status:main"]
    },
    classifiers=[
        "Development Status :: 3 - Alpha",
        "License :: OSI Approved :: Apache Software License",
    ],
    python_requires=">=3.6",
)



import os
import tempfile
from tests.test_base import TestBase
from capanno_utils.repo_config import config
from capanno_utils.content_maps import make_tools_map_dict, make_script_maps
from capanno_utils.status import update_existing_tool_wrapper_status, update_all_tools_to_released


class TestChangeStatus(TestBase):

    def test_update_existing_files_to_released(self):
        tool_map_item = {
            'metadataPath': 'tools/samtools/1.x/samtools_idxstats/samtools-idxstats-metadata.yaml',
            'name': 'idxstats',
            'metadataStatus': 'Released',
            'cwlStatus': 'Incomplete',
            'cwlExists': False,
            'nextflowStatus': 'Incomplete',
            'nextflowExists': False,
            'snakemakeStatus': 'Incomplete',
            'snakemakeExists': True,
            'wdlStatus': 'Incomplete',
            'wdlExists': False,
            'type': 'subtool'} #'TL_ec2a8d_d1.0b'
        with tempfile.TemporaryDirectory() as tmp_dir:
            self.make_empty_tools_dir(tmp_dir)
            self.make_empty_tools_index(tmp_dir)
            update_existing_tool_wrapper_status(tool_map_item, base_dir=self.src_content_dir, update_dir=tmp_dir)
            assert True
        return

    def test_update_all(self):
        """Update all workflow files that exist to released. Make changes in update_dir to keep original unchanged."""
        with tempfile.TemporaryDirectory() as tmp_dir:
            self.copy_all_content_repo_tools(tmp_dir)
            update_all_tools_to_released(base_dir=self.src_content_dir, update_dir=tmp_dir)
            assert True
        return



import os
import unittest
from tests.test_base import TestBase
from capanno_utils.repo_config import config
from ruamel.yaml import safe_load
from capanno_utils.validate_inputs import validate_inputs_for_instance
from capanno_utils.helpers.get_paths import get_tool_instance_path, get_tool_sources

class TestValidateInputs(TestBase):

    # @unittest.skip("Comment out and run smaller tests to isolate problems.")
    def test_validate_tool_inputs_1(self):
        tool_name = 'cat'
        tool_version = '8.x'
        input_hash = '8a6c'
        tool_sources = get_tool_sources(tool_name, tool_version)
        instance_path = get_tool_instance_path(tool_name, tool_version, input_hash)
        validate_inputs_for_instance(instance_path, tool_sources['cwl'])
        return


    # @unittest.skip("Comment out and run smaller tests to isolate problems.")
    def test_validate_tool_inputs_2(self):
        tool_name = 'samtools'
        tool_version = '1.x'
        subtool = 'flagstat'
        instance_hash = '395d'
        tool_sources = get_tool_sources(tool_name, tool_version, subtool_name=subtool, base_dir=self.test_content_dir)
        instance_path = get_tool_instance_path(tool_name, tool_version, instance_hash, subtool_name=subtool, base_dir=self.test_content_dir)
        validate_inputs_for_instance(instance_path, tool_sources['cwl'])
        return
    #
    # # @unittest.skip('')
    # def test_validate_all_tool_inputs(self):
    #     tool_map = config[os.environ.get('CONFIG_KEY')]['content_maps_dir'] / 'tool-maps.yaml'  # Need to change this if want to do tests
    #     with tool_map.open('r') as tm:
    #         tool_map_dict = safe_load(tm)
    #     for tool_identifier, tool_data in tool_map_dict.items():
    #         if tool_data['type'] == 'parent':
    #             continue
    #         elif tool_data['type'] == 'tool':
    #             tool_name = tool_data['name']
    #
    #             raise NotImplementedError
    #     return





from tests.test_base import TestBase
from capanno_utils.helpers.dict_tools import no_clobber_update


test_dict_1 = {'key1': 'value1', 'key2': 'value2', 'key3': 'value3'}
test_dict_2 = {'key4': 'value4', 'key5': 'value5', 'key6': 'value6'}
test_dict_3 = {'key1': 'value1_test3', 'key7': 'value7', 'key4': 'value4_test3'}

class TestDictTools(TestBase):

    def test_no_clobber_update(self):
        updated_dict = no_clobber_update(test_dict_1, test_dict_2)
        self.assertEqual(updated_dict, test_dict_1.update(test_dict_2))
        return

    def test_no_clobber_update_fail(self):
        with self.assertRaises(ValueError):
            no_clobber_update(test_dict_1, test_dict_2)
        return


import io
import pickle
from pathlib import Path
from ruamel.yaml import YAML
from ruamel.yaml.representer import RepresenterError

def dump_yaml(yaml_object, map_object, target):
    try:
        yaml_object.dump(map_object, target)
    except RepresenterError:
        print(f"pickle: {pickle.dumps(map_object)}")
        raise
    return

def dump_dict_to_yaml_output(dict_, output=None):
    yaml = YAML(pure=True)
    yaml.default_flow_style = False
    yaml.indent(mapping=2, sequence=4, offset=2)
    if output:
        return_output = Path(output)
        with return_output.open('w') as yaml_file:
            dump_yaml(yaml, dict_, yaml_file)
    else:
        text_stream = io.StringIO()
        dump_yaml(yaml, dict_, text_stream)
        return_output = text_stream.getvalue()
    return return_output


import tempfile
from pathlib import Path
from urllib.parse import urlparse
from copy import deepcopy
from ruamel.yaml.comments import CommentedMap
from schema_salad.ref_resolver import file_uri, Loader
from schema_salad.jsonld_context import salad_to_jsonld_context
from schema_salad.schema import get_metaschema, validate_doc, collect_namespaces, make_avro, make_avro_schema_from_avro
from capanno_utils.classes.cwl.common_workflow_language import load_document
from capanno_utils.helpers.string_tools import get_shortened_id
from capanno_utils.helpers.dict_tools import get_dict_from_list
from capanno_utils.helpers.file_management import dump_dict_to_yaml_output


class SaladSchemaBase:
    # Could we use schema_salad.schema.get_metaschema() or something instead?
    metaschema_base = {'$base': 'https://w3id.org/cwl/salad#',
                       '$graph': [{'doc': '# Schema\n', 'name': 'Schema', 'type': 'documentation'},
                                  {'doc': ['Salad data types are based on Avro schema declarations.  '
                                           'Refer to the\n'
                                           '[Avro schema declaration '
                                           'documentation](https://avro.apache.org/docs/current/spec.html#schemas) '
                                           'for\n'
                                           'detailed information.\n',
                                           'null: no value',
                                           'boolean: a binary value',
                                           'int: 32-bit signed integer',
                                           'long: 64-bit signed integer',
                                           'float: single precision (32-bit) IEEE 754 floating-point '
                                           'number',
                                           'double: double precision (64-bit) IEEE 754 '
                                           'floating-point number',
                                           'string: Unicode character sequence'],
                                   'name': 'PrimitiveType',
                                   'symbols': ['sld:null',
                                               'xsd:boolean',
                                               'xsd:int',
                                               'xsd:long',
                                               'xsd:float',
                                               'xsd:double',
                                               'xsd:string'],
                                   'type': 'enum'},
                                  {'doc': 'The **Any** type validates for any non-null value.\n',
                                   'docAfter': '#PrimitiveType',
                                   'name': 'Any',
                                   'symbols': ['#Any'],
                                   'type': 'enum'},
                                  {'doc': 'A field of a record.',
                                   'fields': [{'doc': 'The name of the field\n',
                                               'jsonldPredicate': '@id',
                                               'name': 'name',
                                               'type': 'string'},
                                              {'doc': 'A documentation string for this field\n',
                                               'jsonldPredicate': 'rdfs:comment',
                                               'name': 'doc',
                                               'type': 'string?'},
                                              {'doc': 'The field type\n',
                                               'jsonldPredicate': {'_id': 'sld:type',
                                                                   '_type': '@vocab',
                                                                   'refScope': 2,
                                                                   'typeDSL': True},
                                               'name': 'type',
                                               'type': ['PrimitiveType',
                                                        'RecordSchema',
                                                        'EnumSchema',
                                                        'ArraySchema',
                                                        'string',
                                                        {'items': ['PrimitiveType',
                                                                   'RecordSchema',
                                                                   'EnumSchema',
                                                                   'ArraySchema',
                                                                   'string'],
                                                         'type': 'array'}]}],
                                   'name': 'RecordField',
                                   'type': 'record'},
                                  {'fields': {'fields': {'doc': 'Defines the fields of the record.',
                                                         'jsonldPredicate': {'_id': 'sld:fields',
                                                                             'mapPredicate': 'type',
                                                                             'mapSubject': 'name'},
                                                         'type': 'RecordField[]?'},
                                              'type': {'doc': 'Must be `record`',
                                                       'jsonldPredicate': {'_id': 'sld:type',
                                                                           '_type': '@vocab',
                                                                           'refScope': 2,
                                                                           'typeDSL': True},
                                                       'type': {'name': 'Record_symbol',
                                                                'symbols': ['sld:record'],
                                                                'type': 'enum'}}},
                                   'name': 'RecordSchema',
                                   'type': 'record'},
                                  {'doc': 'Define an enumerated type.\n',
                                   'fields': {'symbols': {'doc': 'Defines the set of valid symbols.',
                                                          'jsonldPredicate': {'_id': 'sld:symbols',
                                                                              '_type': '@id',
                                                                              'identity': True},
                                                          'type': 'string[]'},
                                              'type': {'doc': 'Must be `enum`',
                                                       'jsonldPredicate': {'_id': 'sld:type',
                                                                           '_type': '@vocab',
                                                                           'refScope': 2,
                                                                           'typeDSL': True},
                                                       'type': {'name': 'Enum_symbol',
                                                                'symbols': ['sld:enum'],
                                                                'type': 'enum'}}},
                                   'name': 'EnumSchema',
                                   'type': 'record'},
                                  {'fields': {'items': {'doc': 'Defines the type of the array '
                                                               'elements.',
                                                        'jsonldPredicate': {'_id': 'sld:items',
                                                                            '_type': '@vocab',
                                                                            'refScope': 2},
                                                        'type': ['PrimitiveType',
                                                                 'RecordSchema',
                                                                 'EnumSchema',
                                                                 'ArraySchema',
                                                                 'string',
                                                                 {'items': ['PrimitiveType',
                                                                            'RecordSchema',
                                                                            'EnumSchema',
                                                                            'ArraySchema',
                                                                            'string'],
                                                                  'type': 'array'}]},
                                              'type': {'doc': 'Must be `array`',
                                                       'jsonldPredicate': {'_id': 'sld:type',
                                                                           '_type': '@vocab',
                                                                           'refScope': 2,
                                                                           'typeDSL': True},
                                                       'type': {'name': 'Array_symbol',
                                                                'symbols': ['sld:array'],
                                                                'type': 'enum'}}},
                                   'name': 'ArraySchema',
                                   'type': 'record'}],
                       '$namespaces': {'dct': 'http://purl.org/dc/terms/',
                                       'rdf': 'http://www.w3.org/1999/02/22-rdf-syntax-ns#',
                                       'rdfs': 'http://www.w3.org/2000/01/rdf-schema#',
                                       'sld': 'https://w3id.org/cwl/salad#',
                                       'xsd': 'http://www.w3.org/2001/XMLSchema#'}}

    def __init__(self):
        pass


class InputsSchema:
    template_dict = {'$base': 'https://w3id.org/cwl/cwl#',
                     '$namespaces': {'cwl': 'https://w3id.org/cwl/cwl#',
                                     'rdfs': 'http://www.w3.org/2000/01/rdf-schema#',
                                     'sld': 'https://w3id.org/cwl/salad#'},
                     '$graph': [{'$import': 'null'},  # Replace with path to metaschema-base temp file.
                                {'extends': 'sld:PrimitiveType',
                                 'name': 'CWLType',
                                 'symbols': ['cwl:File', 'cwl:Directory'],
                                 'type': 'enum'},
                                {'doc': None,
                                 'docParent': '#CWLType',
                                 'fields': [{'jsonldPredicate': {'_id': '@type', '_type': '@vocab'},
                                             'name': 'class',
                                             'type': {'name': 'File_class',
                                                      'symbols': ['cwl:File'],
                                                      'type': 'enum'}},
                                            {'jsonldPredicate': {'_id': '@id', '_type': '@id'},
                                             'name': 'location',
                                             'type': 'string?'},
                                            {'jsonldPredicate': {'_id': 'cwl:path', '_type': '@id'},
                                             'name': 'path',
                                             'type': 'string?'},
                                            {'jsonldPredicate': 'cwl:basename',
                                             'name': 'basename',
                                             'type': 'string?'},
                                            {'name': 'dirname', 'type': 'string?'},
                                            {'name': 'nameroot', 'type': 'string?'},
                                            {'name': 'nameext', 'type': 'string?'},
                                            {'name': 'checksum', 'type': 'string?'},
                                            {'name': 'size', 'type': 'long?'},
                                            {'jsonldPredicate': 'cwl:secondaryFiles',
                                             'name': 'secondaryFiles',
                                             'type': ['null',
                                                      {'items': ['File', 'Directory'],
                                                       'type': 'array'}]},
                                            {'jsonldPredicate': {'_id': 'cwl:format',
                                                                 '_type': '@id',
                                                                 'identity': True},
                                             'name': 'format',
                                             'type': 'string?'},
                                            {'name': 'contents', 'type': 'string?'}],
                                 'name': 'File',
                                 'type': 'record'},
                                {'fields': [{'jsonldPredicate': {'_id': '@type', '_type': '@vocab'},
                                             'name': 'class',
                                             'type': {'name': 'Directory_class',
                                                      'symbols': ['cwl:Directory'],
                                                      'type': 'enum'}},
                                            {'jsonldPredicate': {'_id': '@id', '_type': '@id'},
                                             'name': 'location',
                                             'type': 'string?'},
                                            {'jsonldPredicate': {'_id': 'cwl:path', '_type': '@id'},
                                             'name': 'path',
                                             'type': 'string?'},
                                            {'jsonldPredicate': 'cwl:basename',
                                             'name': 'basename',
                                             'type': 'string?'},
                                            {'jsonldPredicate': {'_id': 'cwl:listing'},
                                             'name': 'listing',
                                             'type': ['null',
                                                      {'items': ['File', 'Directory'],
                                                       'type': 'array'}]}],
                                 'name': 'Directory',
                                 'type': 'record'},
                                {'documentRoot': True,
                                 'fields': None,
                                 'name': 'InputsField',
                                 'type': 'record'}],
                     }

    def __init__(self, cwl_doc):
        if isinstance(cwl_doc, (str, Path)):
            self.cwl_path = cwl_doc
            cwl_document = load_document(str(self.cwl_path))
        else:  # assume cwl_doc is CommandLineTool object.
            cwl_document = cwl_doc
        self._cwl_inputs = cwl_document.inputs
        self._cwl_schema_def_requirement = cwl_document.get_schema_def_requirement()

    @property
    def cwl_inputs(self):
        return self._cwl_inputs

    @property
    def cwl_schema_def_requirement(self):
        return self._cwl_schema_def_requirement # set in __init__

    def validate_inputs(self, document_path):
        """

        :param document_path:
        :return:
        """
        with tempfile.NamedTemporaryFile(prefix='metaschema_base', suffix='.yml') as tmp_meta_base:
            dump_dict_to_yaml_output(SaladSchemaBase.metaschema_base, tmp_meta_base.name)  # Convenient to put it in tmp directory where inputs schema will live.
            with tempfile.NamedTemporaryFile(prefix='inputs_schema', suffix='.yml') as tmp:
                self._make_inputs_schema_file(tmp_meta_base.name, tmp.name)
                self._schema_salad_validate(tmp.name, document_path)
        return

    def _make_inputs_schema_dict(self):
        """
        Make the schema from inputs to validate job file with.
        :return (dict):
        """
        inputs_fields = {}
        for input in self.cwl_inputs:  # inputs is a list of CommandInputParameter
            inputs_fields[get_shortened_id(input.id)] = {
                'type': input._handle_input_type_field(self.cwl_schema_def_requirement)}

        schema_dict = deepcopy(InputsSchema.template_dict)
        _, inputs_field_index = get_dict_from_list(schema_dict['$graph'], 'name', 'InputsField')
        schema_dict['$graph'][inputs_field_index]['fields'] = inputs_fields
        return schema_dict

    def _make_inputs_schema_file(self, metaschema_path, out_file):
        """

        :param metaschema_path:
        :param out_file:
        :return:
        """
        schema_dict = self._make_inputs_schema_dict()
        import_dict, import_index = get_dict_from_list(schema_dict['$graph'], '$import', 'null')
        schema_dict['$graph'][import_index] = {'$import': metaschema_path}
        dump_dict_to_yaml_output(schema_dict, out_file)
        return

    def _schema_salad_validate(self, schema_path, document_path):
        '''
        Adapted from schema_salad main().
        :param schema_path:
        :param document_path:
        :return:
        '''

        strict_foreign_properties = False
        strict = True
        metaschema_names, metaschema_doc, metaschema_loader = get_metaschema()
        schema_uri = str(schema_path)
        if not (urlparse(schema_uri)[0] and urlparse(schema_uri)[0] in ['http', 'https', 'file']):
            schema_uri = file_uri(schema_uri)
        schema_raw_doc = metaschema_loader.fetch(schema_uri)

        schema_doc, schema_metadata = metaschema_loader.resolve_all(schema_raw_doc, schema_uri)

        # Validate schema against metaschema
        validate_doc(metaschema_names, schema_doc, metaschema_loader, True)

        # Get the json-ld context and RDFS representation from the schema
        metactx = collect_namespaces(schema_metadata)
        if "$base" in schema_metadata:
            metactx["@base"] = schema_metadata["$base"]

        (schema_ctx, rdfs) = salad_to_jsonld_context(
            schema_doc, metactx)

        # Create the loader that will be used to load the target document.
        document_loader = Loader(schema_ctx, skip_schemas=False)

        # Make the Avro validation that will be used to validate the target
        # document

        avsc_obj = make_avro(schema_doc, document_loader)

        avsc_names = make_avro_schema_from_avro(avsc_obj)

        # Load target document and resolve refs
        uri = str(document_path)
        document, doc_metadata = document_loader.resolve_ref(uri, strict_foreign_properties=strict_foreign_properties,
                                                             checklinks=False)  # This is what's getting us around file link checking.

        validate_doc(avsc_names, document, document_loader, strict=strict,
                     strict_foreign_properties=strict_foreign_properties)

        return

    @staticmethod
    def _make_input_value_field(command_input_parameter, schema_def_requirement):
        template_param_value, comment = command_input_parameter.make_input_value_field(schema_def_requirement)
        return template_param_value, comment

    def make_template(self):
        """
        Make a cwl job file template.
        Inspired by cwl-tool --make-template (calls cwl-tool.main.generate_input_template https://github.com/common-workflow-language/cwltool/blob/main/cwltool/main.py)
        :return:
        """
        template = CommentedMap()
        for input in self.cwl_inputs:
            input_name = get_shortened_id(input.id)
            template_param_value, comment = self._make_input_value_field(input, self._cwl_schema_def_requirement)
            template.insert(0, input_name, template_param_value, comment)
        return template

    def make_template_file(self, template_file):
        template = self.make_template()
        dump_dict_to_yaml_output(template, template_file)
        return




import argparse
import sys
from pathlib import Path
from capanno_utils.repo_config import content_repo_name
from capanno_utils.helpers.get_paths import get_dir_type_from_path
from capanno_utils.helpers.file_management import dump_dict_to_yaml_output
from capanno_utils.content_maps import *


def get_parser():
    parser = argparse.ArgumentParser(description=f"Make map of content in {content_repo_name}")
    parser.add_argument('path', type=Path,
                        help='Provide the directory path to make a map of.')
    parser.add_argument('output_path', type=Path, nargs='?', default=None,
                        help='Provide a file to output the map to.')
    parser.add_argument('-p', '--root-repo-path', dest='root_path', type=Path, default=Path.cwd(),
                        help="Specify the root path of your content repo if it is not the current working directory.")
    parser.add_argument('-q', '--quiet', dest='quiet', action='store_true', help="Silence messages to stdout")
    parser.add_argument('--include-exists', dest='exists', action='store_true',
                        help="Include whether workflow files exist for tools in the output. Currently for tools only.")

    return parser


def main(argsl=None):
    if argsl is None:
        argsl = sys.argv[1:]

    parser = get_parser()
    args = parser.parse_args(argsl)
    base_dir = args.root_path.resolve()
    exists = args.exists
    if args.path.is_absolute():
        full_path = args.path
    else:
        full_path = base_dir / args.path

    if not full_path.is_dir():
        raise ValueError(f"{full_path} is not a directory")

    base_type, dir_type = get_dir_type_from_path(full_path, content_root_repo_name=base_dir.name)

    if base_type == 'base_dir':  # Content source root provided.
        # import pdb; pdb.set_trace()
        output_map = make_master_map_dict(base_dir=base_dir, specify_exists=exists)
    elif base_type == 'tool':
        if dir_type == 'base_dir':  # Base tools dir.
            output_map = make_tools_map_dict(base_dir=base_dir, specify_exists=exists)
        elif dir_type == 'tool_dir':
            output_map = make_main_tool_map(tool_name=full_path.name, base_dir=base_dir)
        elif dir_type == 'version_dir':
            print('version_dir')
            output_map = make_tool_version_dir_map(tool_name=full_path.parts[-2], tool_version=full_path.name,
                                                   base_dir=base_dir)
        elif dir_type == 'common_dir':
            output_map = make_tool_common_dir_map(tool_name=full_path.parts[-3], tool_version=full_path.parts[-2],
                                                  base_dir=base_dir)
        elif dir_type == 'subtool_dir':
            subtool_name = full_path.name.split('_')[-1]
            output_map = make_subtool_map(tool_name=full_path.parts[-3], tool_version=full_path.parts[-2],
                                          subtool_name=subtool_name, base_dir=base_dir)
        else:
            raise ValueError
    elif base_type == 'script':
        if dir_type == 'base_dir':
            output_map = make_scripts_map_dict(base_dir=base_dir)
        elif dir_type == 'group_dir':
            output_map = make_group_script_map(group_name=full_path.name, base_dir=base_dir)
        elif dir_type == 'project_dir':
            output_map = make_project_script_map(group_name=full_path.parts[-2], project_name=full_path.name,
                                                 base_dir=base_dir)
        elif dir_type == 'version_dir':
            output_map = make_script_version_map(group_name=full_path.parts[-3], project_name=full_path.parts[-2],
                                                 version_name=full_path.name, base_dir=base_dir)
        # elif dir_type == 'common_dir':  # Doesn't specify whole object item
        elif dir_type == 'scripts_dir':
            output_map = make_script_map(group_name=full_path.parts[-4], project_name=full_path.parts[-3],
                                         version_name=full_path.parts[-2], script_name=full_path.name,
                                         base_dir=base_dir)
        else:
            raise ValueError
    elif base_type == 'workflow':
        raise NotImplementedError
    else:
        raise ValueError
    return dump_dict_to_yaml_output(output_map, args.output_path)


if __name__ == '__main__':
    sys.exit(main(sys.argv[1:]))



import logging
import os
from pathlib import Path
from urllib.parse import urlparse
from schema_salad.exceptions import ValidationException
from schema_salad.ref_resolver import file_uri
from cwltool.load_tool import resolve_and_validate_document, fetch_document
from cwltool.main import main as cwl_tool


def validate_cwl_doc_main(cwl_doc_path):
    """
    Not currently used. Calls the main function of cwltool with validation parameters. Does a lot of extra stuff.
    :param cwl_doc_path:
    :return:
    """
    stream_handler = logging.StreamHandler()
    stream_handler.setLevel(logging.ERROR)
    cwl_doc_path = str(cwl_doc_path)
    rv = cwl_tool(argsl=['--validate', '--disable-color', cwl_doc_path], logger_handler=stream_handler)
    if rv != 0:
        raise ValidationException(f"cwltool did not return a return value of 0 for {cwl_doc_path}")
    return


def validate_cwl_doc(cwl_doc):
    """
    This is adapted from cwltool.main.main and avoids the unnecessary stuff by using cwltool.main.main directly.
    :param cwl_doc_path:
    :return:
    """
    if isinstance(cwl_doc, (Path, str)):  # Can also be CWLObjectType
        cwl_doc = str(cwl_doc)
        if not (urlparse(cwl_doc)[0] and urlparse(cwl_doc)[0] in ['http', 'https', 'file']):
            cwl_doc = file_uri(os.path.abspath(cwl_doc))
    loading_context, workflow_object, uri = fetch_document(cwl_doc)
    resolve_and_validate_document(loading_context, workflow_object, uri)
    return



""" Get projects from https://github.com/common-workflow-library/bio-cwl-tools
    make sure this repo is added as a submodule so we can just parse it locally instead of getting bogged down in scraping
    git submodule add git@github.com:common-workflow-library/bio-cwl-tools.git bio-cwl-tools-submodule
    from capanno_utils.helpers.import_repo import bioCwl;mybiocwl=bioCwl();mybiocwl.processDir()
"""

#import cwl_utils.parser_v1_0 as parser
from capanno_utils.classes.cwl import common_workflow_language as pycwl
import os
import re
import sys
import glob
from abc import abstractmethod
import ruamel.yaml
from ruamel.yaml.comments import CommentedMap, CommentedSeq
from ruamel.yaml.compat import string_types, ordereddict
import logging, sys
import subprocess
from typing import Optional
from capanno_utils.add.add_tools import add_tool, add_subtool
from capanno_utils.helpers.get_paths import get_tool_common_dir, main_tool_subtool_name, get_tool_metadata, get_tool_dir
from capanno_utils.helpers.get_paths import get_tool_sources, get_cwl_script, main_tool_subtool_name
from capanno_utils.classes.cwl.common_workflow_language import load_document

logging.basicConfig(stream=sys.stderr)
logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)


class repoImporter(object):
    """Import cwl tools and workflows from a third-party repo"""
    def __init__(
            self,
            *,
            path: str,
            name: Optional[str] = None,
            identifier: Optional[str] = None,
            commithash: Optional[str] = None,
            denylist: Optional[list] = None
    ) -> None:
        """Create some generic repoImporter object
        :param path: local path to the repo (usually a git subclone e.g. bio-cwl-tools-submodule)
        :param name: name of this import ("July_refresh")
        """

        #import the git info
        #git submodule
        #61ffac1862822f08dc20b6f8e2f22634b986b0bc bio-cwl-tools-submodule (heads/release)

        #this might be hard to parse consistently
        #git submodule foreach 'git remote show origin'
        # Entering 'bio-cwl-tools-submodule'
        # * remote origin
        #   Fetch URL: git@github.com:common-workflow-library/bio-cwl-tools.git
        #   Push  URL: git@github.com:common-workflow-library/bio-cwl-tools.git
        self.path=path
        self.abspath=os.path.join(os.path.dirname(__file__),"../../",self.path)
        self.name=name
        self.identifier=identifier
        self.commithash = commithash
        self.denylist = denylist
        process = subprocess.Popen(['git', 'submodule'],
                                   stdout=subprocess.PIPE,
                                   stderr=subprocess.PIPE)
        stdout, stderr = process.communicate()

        m = re.search(' ([a-z0-9]+)', str(stdout))
        # TODO: if there are multiple submodules then it won't work
        self.commithash=m[1]

    @abstractmethod
    def getCwlInfos(self) -> None:
        pass

    @abstractmethod
    def getCwlInfo(self, tool: str, subtoolcwl: str) -> dict:
        pass

class bioSnakemakeWrappers(repoImporter):
    def __init__(
            self,
            *,
            path: str = "snakemake-wrappers/bio",
            denylist: Optional[list] = []
    ) -> None:
        super().__init__(path=path,denylist=denylist)

    def processDir(self) -> None:
        logger.debug("processDir")
        #for tool in [os.path.basename(x[1]) for x in os.walk(self.path)]:
        for tool in os.listdir(self.abspath):
            logger.info("potential tool:{}".format(tool))
            logger.debug("potential tool:{}".format(tool))
            if tool not in self.denylist and os.path.isdir(os.path.join(self.path,tool)) and not tool.startswith('.'):
                logger.info("accepted:{}".format(tool))
                self.processTool(tool)

    def processTool(self, tool, addSubtoolsToTool=True, no_clobber=True) -> None:
        tooldict = {}
        tooldict[tool] = {}
        tooldict[tool]['subtools'] = []

        #TODO: consider using the x[2] from os.walk
        versions_reported = []
        dockers_reported = []
        biotools_reported = []
        for subtoolsource in glob.glob(self.path+"/"+tool+'/wrapper.py'):
            if os.path.basename(subtoolsource) not in self.denylist:
                logger.info("\tsub:{}".format(os.path.basename(subtoolsource)))
                subtooldict=self.getSMWrapInfo(tool,str(os.path.join(os.path.basename(subtoolsource),))
                tooldict[tool]['subtools']+=[subtoolsource]

                for k,v in subtooldict.items():
                    logger.info("{}:{}".format(k, v))
                    #can we figure out a consensus for the version?
                    if k=='version' and v is not None:
                        versions_reported+=[v]
                    elif k=='docker' and v is not None:
                        dockers_reported+=[v]
                    elif k=='biotools' and v is not None:
                        biotools_reported+=[v]


        if len(set(dockers_reported)) > 1:
            logger.error('Observing multiple docker images in the tool {}: {}'.format(tool, dockers_reported))
            tooldict[tool]['docker'] = dockers_reported[0]
        elif len(dockers_reported) == 0:
            logger.error('Observing no docker images in the tool {}'.format(tool))
        else:
            tooldict[tool]['docker'] = dockers_reported[0]

        if len(set(versions_reported)) > 1:
            logger.error(
                'Observing multiple versions in the tool {}: {}'.format(tool, versions_reported))
        elif len(versions_reported) == 0:
            logger.info('Observing no versions in the tool {}'.format(tool))

        if len(set(biotools_reported)) >= 1:
            tooldict[tool]['biotools'] = biotools_reported[0]
        else:
            tooldict[tool]['biotools'] = None

        # is there a docker version we can use?
        # quay.io/biocontainers/picard:2.22.2--0 should really be 2.22.2
        if len(dockers_reported) > 0:
            m = re.search('(\\S+):v?([.0-9]+)', tooldict[tool]['docker'])
            if m is not None:
                tooldict[tool]['version']=m[2]
                versions_reported+=[m[2]]
        #else:
        #    tooldict[tool]['version']=versions_reported[0]

        if len(versions_reported) == 0:
            raise RuntimeError("Can't find version for {}".format(tool))
        else:
            tooldict[tool]['version'] = versions_reported[0]
        logger.info("addtool loop")
        for tool in tooldict.keys():
            subtoolnames = []
            if len(tooldict[tool]['subtools'])>1:
                # TODO: has_primary could be true? we need to figure this out
                has_primary = False
                for subtool in tooldict[tool]['subtools']:
                    subtoolnames += [subtool['name']]
            else:
                has_primary = True
                subtoolnames=main_tool_subtool_name
            # TODO: study the existing tool directory and try not to clobber other versions
            logger.info("adding tool: {} version: {} biotools: {} subtoolnames: {}".format(tool,tooldict[tool]['version'],tooldict[tool]['biotools'],subtoolnames))
            if addSubtoolsToTool:
                add_tool(tool, version_name=tooldict[tool]['version'],subtool_names=[],biotools_id=tooldict[tool]['biotools'], no_clobber = no_clobber)
            else:
                add_tool(tool, version_name=tooldict[tool]['version'], subtool_names=subtoolnames, biotools_id=tooldict[tool]['biotools'],
                                 has_primary=has_primary, init_cwl = False, no_clobber = no_clobber) #, init_cwl=args.init_cwl, root_repo_path=args.root_path)
            #add_tool visits the subtools
            if addSubtoolsToTool:
                for subtool in tooldict[tool]['subtools']:
                    logger.info("adding subtool {}".format(subtool['name']))
                    #add_content_main(['-p', tmp_dir, 'subtool', tool_name, tool_version, subtool_name, '-u', '--init-cwl', cwl_url])

                    #bio-cwl tools has the habit of naming tools like picard_MarkDuplicates.cwl
                    if subtool['name'].startswith(tool):
                        subtool_name = subtool['name'].replace(tool+'_','')
                    else:
                        subtool_name = subtool['name']

                    if subtool['version'] is None:
                        tool_version = tooldict[tool]['version']
                    else:
                        tool_version = subtool['version']
                    logger.info("adding subtool: {} version: {} path: {}".format(subtool_name, subtool['version'],
                                                                                 subtool['path']))
                    add_subtool(tool_name=tool, tool_version=tool_version, subtool_name=subtool_name,init_cwl=subtool['path'], update_featureList=True, no_clobber = no_clobber) #root_repo_path=Path.cwd(),update_featureList=False, init_cwl=subtoolcwl)


    def getCwlInfo(self, tool: str, subtoolcwl: str, acceptClassYaml = False) -> dict:
        """
        Fetch vital information about a subtool
        :param tool: the tool e.g. bamtools
        :param subtoolcwl: the subtool cwl basename e.g. bamtools_stats.cwl
        :param acceptClassYaml: allow -class style descriptors
        :return: dict with tool_name, version_name, subtool name
        """
        filename=os.path.join(self.abspath,tool,subtoolcwl)
        version = docker = biotools = None
        with open(filename) as fp:
            result = ruamel.yaml.main.round_trip_load(fp)
            #result_pycwl = pycwl.load_document(fp)
            if 'hints' in result:
                sftreq = None


                for hintkey in result['hints']: # hintkey = ordereddict([('class', 'DockerRequirement'), ('dockerPull', 'quay.io/biocontainers/picard:2.22.2--0')])
                    if 'class' in hintkey:
                        if not acceptClassYaml:
                            raise ValueError("No -class style Yaml accepted for tool info {} {}".format(tool, subtoolcwl))
                        if hintkey['class']=='SoftwareRequirement':
                            sftreq = hintkey['packages']
                        elif hintkey['class'] == 'DockerRequirement':
                            try:
                                docker = hintkey['dockerPull']
                                logger.debug("docker:{}".format(docker))
                            except KeyError as e:
                                logger.debug("keyerror:{}".format(e))
                    elif hintkey == 'SoftwareRequirement':
                        sftreq = result['hints']['SoftwareRequirement']['packages'][tool]
                    elif hintkey == 'DockerRequirement':
                        # see picard_AddOrReplaceReadGroups
                        # import pdb
                        # pdb.set_trace()
                        if 'dockerPull' in result['hints']['DockerRequirement']:
                            docker = result['hints']['DockerRequirement']['dockerPull']
                        else:
                            # graph-genome-segmentation component_segmentation.cwl
                            docker = result['hints']['DockerRequirement']['dockerFile']
                    if sftreq is not None:
                        logger.info("SoftwareRequirement found")
                        try:
                            version=result['hints']['SoftwareRequirement']['packages'][tool]['version'][0]
                            logger.info("version:{}".format(version))
                        except KeyError as e:
                            logger.debug("keyerror:{}".format(e))
                        except IndexError as e:
                            logger.debug("indexerror:{}".format(e))
                        try:
                            specs=result['hints']['SoftwareRequirement']['packages'][tool]['specs'][0]
                            if 'biotools' in specs or 'bio.tools' in specs:
                                #http://identifiers.org/biotools/gatk
                                #https://bio.tools/picard_arrg
                                m = re.search('bio.?tools/(.+)', specs)
                                biotools = m[1]
                        except KeyError as e:
                            logger.debug("keyerror:{}".format(e))
                        except IndexError as e:
                            logger.debug("indexerror:{}".format(e))
            if 'requirements' in result:
                for req in result['requirements']:
                    if 'dockerPull' in req:
                        docker=req['dockerPull']
                        logger.debug("direct dockerpull:{}".format(subtoolcwl))
                    #see Lancet
                    # ('requirements', ordereddict([('DockerRequirement', ordereddict([('dockerPull', 'sinaiiidgst/lancet:latest')])), ('InlineJavascriptRequirement', ordereddict())])), ('inputs', ordereddict([('TumorInput', ordereddict([('type', 'File'), ('inputBinding', ordereddict([('prefix', '--tumor')])), ('secondaryFiles', ['.bai'])])), ('NormalInput', ordereddict([('type', 'File'), ('inputBinding', ordereddict([('prefix', '--normal')])), ('secondaryFiles', ['.bai'])])), ('Reference', ordereddict([('type', 'File'), ('format', 'edam:format_1929'), ('inputBinding', ordereddict([('prefix', '--ref')]))])), ('GenomicRegion', ordereddict([('type', ['null', ordereddict([('type', 'record'), ('name', 'GenomicRegion'), ('fields', ordereddict([('Chromosome', ordereddict([('type', 'string')])), ('RegionStart', ordereddict([('type', 'int')])), ('RegionEnd', ordereddict([('type', 'int')])), ('GenomicRegion', ordereddict([('type', 'string'), ('inputBinding', ordereddict([('prefix', '--reg'), ('valueFrom', '$(inputs.GenomicRegion.Chromosome + ":" + inputs.GenomicRegion.RegionStart + "-" + inputs.GenomicRegion.RegionEnd)')]))]))]))])])])), ('BedFile', ordereddict([('type', 'File?'), ('inputBinding', ordereddict([('prefix', '--bed')]))])), ('MinKmer', ordereddict([('type', 'int?'), ('inputBinding', ordereddict([('prefix', '--min-k')]))])), ('MaxKmer', ordereddict([('type', 'int?'), ('inputBinding', ordereddict([('prefix', '--max-k')]))])), ('TrimLowQuality', ordereddict([('type', 'int?'), ('inputBinding', ordereddict([('prefix', '--trim-lowqual')]))])), ('MinBaseQual', ordereddict([('type', 'int?'), ('inputBinding', ordereddict([('prefix', '--min-base-qual')]))])), ('QualityRange', ordereddict([('type', 'string?'), ('inputBinding', ordereddict([('prefix', '--quality-range')]))])), ('MinMapQual', ordereddict([('type', 'int?'), ('inputBinding', ordereddict([('prefix', '--min-map-qual')]))])), ('ASXSDifMax', ordereddict([('type', 'int?'), ('inputBinding', ordereddict([('prefix', '--max-as-xs-diff')]))])), ('MaxTipLen', ordereddict([('type', 'int?'), ('inputBinding', ordereddict([('prefix', '--tip-len')]))])), ('MinCovThres', ordereddict([('type', 'int?'), ('inputBinding', ordereddict([('prefix', '--cov-thr')]))])), ('MinCovRatio', ordereddict([('type', 'float?'), ('inputBinding', ordereddict([('prefix', '--cov-ratio')]))])), ('LowCovThres', ordereddict([('type', 'int?'), ('inputBinding', ordereddict([('prefix', '--low-cov')]))])), ('MaxAvgCov', ordereddict([('type', 'int?'), ('inputBinding', ordereddict([('prefix', '--max-avg-cov')]))])), ('WinSize', ordereddict([('type', 'int?'), ('inputBinding', ordereddict([('prefix', '--window-size')]))])), ('Padding', ordereddict([('type', 'int?'), ('inputBinding', ordereddict([('prefix', '--padding')]))])), ('DFSLimit', ordereddict([('type', 'int?'), ('inputBinding', ordereddict([('prefix', '--dfs-limit')]))])), ('MaxIndelLen', ordereddict([('type', 'int?'), ('inputBinding', ordereddict([('prefix', '--max-inde-len')]))])), ('MaxMismatch', ordereddict([('type', 'int?'), ('inputBinding', ordereddict([('prefix', '--max-mismatch')]))])), ('Threads', ordereddict([('type', 'int?'), ('inputBinding', ordereddict([('prefix', '--num-threads')]))])), ('NodeStrLen', ordereddict([('type', 'int?'), ('inputBinding', ordereddict([('prefix', '--node-str-len')]))])), ('MinAltCountTumor', ordereddict([('type', 'int?'), ('inputBinding', ordereddict([('prefix', '--min-alt-count-tumor')]))])), ('MaxAltCountNormal', ordereddict([('type', 'int?'), ('inputBinding', ordereddict([('prefix', '--max-alt-count-normal')]))])), ('MinVAFTumor', ordereddict([('type', 'float?'), ('inputBinding', ordereddict([('prefix', '--min-vaf-tumor')]))])), ('MaxVAFNormal', ordereddict([('type', 'float?'), ('inputBinding', ordereddict([('prefix', '--max-vaf-normal')]))])), ('MinCovTumor', ordereddict([('type', 'int?'), ('inputBinding', ordereddict([('prefix', '--min-coverage-tumor')]))])), ('MaxCovTumor', ordereddict([('type', 'int?'), ('inputBinding', ordereddict([('prefix', '--max-coverage-tumor')]))])), ('MinCovNormal', ordereddict([('type', 'int?'), ('inputBinding', ordereddict([('prefix', '--min-coverage-normal')]))])), ('MaxCovNormal', ordereddict([('type', 'int?'), ('inputBinding', ordereddict([('prefix', '--max-coverage-normal')]))])), ('MinPhredFisher', ordereddict([('type', 'float?'), ('inputBinding', ordereddict([('prefix', '--min-phred-fisher')]))])), ('MinPhredFisherSTR', ordereddict([('type', 'float?'), ('inputBinding', ordereddict([('prefix', '--min-phred-fisher-str')]))])), ('MinStrandBias', ordereddict([('type', 'float?'), ('inputBinding', ordereddict([('prefix', '--min-strand-bias')]))])), ('MaxUnitLen', ordereddict([('type', 'int?'), ('inputBinding', ordereddict([('prefix', '--max-unit-length')]))])), ('MinReportUnit', ordereddict([('type', 'int?'), ('inputBinding', ordereddict([('prefix', '--min-report-unit')]))])), ('MinReportLen', ordereddict([('type', 'int?'), ('inputBinding', ordereddict([('prefix', '--min-report-len')]))])), ('DistFrSTR', ordereddict([('type', 'int?'), ('inputBinding', ordereddict([('prefix', '--dist-from-str')]))])), ('ActRegOff', ordereddict([('type', 'boolean?'), ('inputBinding', ordereddict([('prefix', '--active-region-off')]))])), ('KmerRecov', ordereddict([('type', 'boolean?'), ('inputBinding', ordereddict([('prefix', '--kmer-recovery')]))])), ('PrintGraph', ordereddict([('type', 'boolean?'), ('inputBinding', ordereddict([('prefix', '--print-graph')]))])), ('Verbose', ordereddict([('type', 'boolean?'), ('inputBinding', ordereddict([('prefix', '--verbose')]))]))])), ('baseCommand', 'lancet'), ('outputs', ordereddict([('vcf', ordereddict([('format', 'edam:format_3016'), ('type', 'stdout')]))])), ('stdout', 'lancet-out.vcf'), ('$namespaces', ordereddict([('edam', 'http://edamontology.org/')])), ('$schemas', ['http://edamontology.org/EDAM_1.18.owl'])])
                    if 'DockerRequirement' in req:
                        logger.debug("dockerrequirement.dockerpull dockerpull:{}".format(subtoolcwl))
                        docker=result['requirements']['DockerRequirement']['dockerPull']

        name=subtoolcwl.replace('.cwl','')
        if not all([name, subtoolcwl, version, docker, biotools]):
            logger.error('missing one of name:{} cwl:{} version:{} docker:{} biotools:{}'.format(name,subtoolcwl,version,docker,biotools))
            #raise Exception('missing one of name:{} cwl:{} version:{} docker:{}'.format(name,subtoolcwl,version,docker))
        return({'name':name,'basename':subtoolcwl,'path':filename,'version':version,'docker':docker,'biotools':biotools})



# -*- coding: utf-8 -*-

"""Test the repo importer"""

import unittest

import cwl_utils.parser_v1_0 as parser
import os
import re
import sys
import glob
from abc import abstractmethod
import ruamel.yaml
from ruamel.yaml.comments import CommentedMap, CommentedSeq
from ruamel.yaml.compat import string_types, ordereddict
import logging, sys
import subprocess
from typing import Optional
from capanno_utils.add.add_tools import add_tool, add_subtool
from capanno_utils.helpers.get_paths import get_tool_common_dir, main_tool_subtool_name, get_tool_metadata, get_tool_dir

from capanno_utils.helpers.import_repo import bioCwl

class TestImport(unittest.TestCase):
    def test_import_noversion(self):
        mybio = bioCwl()
        lancetres=mybio.getCwlInfo('Lancet','Lancet.cwl')
        assert(lancetres['docker']=='truwl/lancet:latest')

    def test_import(self):
        mybio = bioCwl()
        allres=mybio.getCwlInfos()
